package aggSim;

import static muvitorkit.ui.utils.GenericUtils.inverseLookup;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

import org.eclipse.core.runtime.Assert;

import roneditor.model.RuleChecker;
import ronmodel.LHS;
import ronmodel.Mapping;
import ronmodel.NAC;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ONPlace;
import ronmodel.ObjectNet;
import ronmodel.RHS;
import ronmodel.RonmodelFactory;
import ronmodel.Rule;
import agg.attribute.AttrType;
import agg.attribute.handler.AttrHandler;
import agg.attribute.handler.impl.javaExpr.JexHandler;
import agg.parser.CriticalPair;
import agg.parser.ExcludePairContainer;
import agg.util.Pair;
import agg.xt_basis.Arc;
import agg.xt_basis.Completion_InjCSP;
import agg.xt_basis.Completion_NAC;
import agg.xt_basis.DefaultGraTraImpl;
import agg.xt_basis.GraGra;
import agg.xt_basis.GraTra;
import agg.xt_basis.Graph;
import agg.xt_basis.GraphObject;
import agg.xt_basis.Match;
import agg.xt_basis.Morphism;
import agg.xt_basis.Node;
import agg.xt_basis.OrdinaryMorphism;
import agg.xt_basis.Step;
import agg.xt_basis.Type;
import agg.xt_basis.TypeException;
import agg.xt_basis.TypeSet;

public class Grammar {
	static private final TypeSet typeSet = new TypeSet();
	static private final Type placeType = typeSet.createType(true);
	static private final Type tokenType = typeSet.createType(true);
	static private final Type arcType = typeSet.createType(true);
	static private final Type tokenArcType = typeSet.createType(true);
	static private final AttrHandler handler = new JexHandler();

	static {
		// create an AGG type graph with Place, Transition, ArcPT and ArcTP.
		// create place node type
		placeType.setStringRepr("Place");
		placeType
				.setAdditionalRepr(":RECT:java.awt.Color[r=0,g=0,b=0]::[NODE]:");
		final AttrType attr = placeType.getAttrType();
		attr.addMember(handler, "String", "name");
		// attr.addMember(handler, "int", "token");
		// attr.addMember(handler, "int", "capacity");

		tokenType.setStringRepr("Token");
		tokenType
				.setAdditionalRepr(":RECT:java.awt.Color[r=0,g=0,b=255]::[NODE]:");
		tokenType.getAttrType().addMember(handler, "int", "number");

		// create arcPT edge type
		arcType.setStringRepr("");
		arcType.setAdditionalRepr(":SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:");
		arcType.getAttrType().addMember(handler, "int", "weight");

		// create arcPT edge type
		tokenArcType.setStringRepr("");
		tokenArcType
				.setAdditionalRepr(":SOLID_LINE:java.awt.Color[r=0,g=0,b=255]::[EDGE]:");
	}

	private final GraGra aggGrammar = new GraGra(typeSet);

	private final HashMap<Rule, agg.xt_basis.Rule> ronRule2aggRule = new HashMap<Rule, agg.xt_basis.Rule>();
	private final HashMap<ObjectNet, Graph> objectNet2graph = new HashMap<ObjectNet, Graph>();
	private final HashMap<ObjectNet, HashMap<ONNode, Node>> objectNet2NodeMap = new HashMap<ObjectNet, HashMap<ONNode, Node>>();
	private final HashMap<ObjectNet, HashMap<ONArc, Arc>> objectNet2ArcMap = new HashMap<ObjectNet, HashMap<ONArc, Arc>>();

	public Grammar() {
		aggGrammar
				.setGraTraOptions(new Completion_NAC(new Completion_InjCSP()));
		final Graph graph = aggGrammar.getGraph();
		aggGrammar.removeGraph(graph);
	}

	/**
	 * Checks whether the specified rule is applicable to the object net.
	 * 
	 * @param onRule
	 *            An object net rule that should be applied to an object net.
	 * @param net
	 *            The object net the rule should be applied to.
	 * @return true, if the rule is applicable to the object net
	 */
	public boolean isApplicable(final Rule onRule, final ObjectNet net) {
		return isApplicable(null, onRule, net);
	}

	/**
	 * Checks whether the specified rule is applicable to the object net.
	 * 
	 * @param mapping
	 *            A mapping between ONNodes in the LHS and the object net the
	 *            rule should be applied to.
	 * @param onRule
	 *            An object net rule that should be applied to an object net.
	 * @param net
	 *            The object net the rule should be applied to.
	 * @return true, if the rule is applicable to the object net
	 */
	public boolean isApplicable(final NetMorphism mapping, final Rule onRule,
			final ObjectNet net) {

		Assert.isTrue(
				mapping == null || mapping.getDomain() == net
						&& mapping.getCodomain() == onRule.getLhs(),
				"Wrong NetMorphism type!");

		if (!RuleChecker.isRuleOk(onRule)) {
			return false;
		}

		final Graph aggGraph = getAGGGraph(net);
		final agg.xt_basis.Rule aggRule = getAGGRule(onRule);

		aggGrammar.resetGraph(aggGraph);
		final Match aggMatch = aggGrammar.createMatch(aggRule);
		mapping2Morphism(mapping, aggMatch);

		boolean result = aggMatch.nextCompletion();
		result = result && aggMatch.isValid();

		// aggGrammar.save("C:\\Eclipse\\test.ggx");

		return result;
	}

	public boolean applyRule(final Rule ronRule, final ObjectNet objectNet) {
		return applyRule(null, ronRule, objectNet);
	}

	public boolean applyRule(final NetMorphism mapping, final Rule ronRule,
			final ObjectNet objectNet) {
		if (!isApplicable(mapping, ronRule, objectNet)) {
			return false;
		}

		final Graph graph = getAGGGraph(objectNet);
		final agg.xt_basis.Rule aggRule = getAGGRule(ronRule);

		aggGrammar.resetGraph(graph);
		final GraTra transformation = new DefaultGraTraImpl();
		transformation.setGraGra(aggGrammar);

		final Match aggMatch = transformation.createMatch(aggRule);
		mapping2Morphism(mapping, aggMatch);
		aggMatch.nextCompletion();
		final Morphism coMatch;
		try {
			coMatch = new Step().execute(aggMatch);
			if (coMatch == null) {
				return false;
			}
		} catch (final TypeException e) {
			return false;
		}

		final HashMap<ONNode, Node> objectNode2graphNodeMap = objectNet2NodeMap
				.get(objectNet);

		final List<Node> preservedNodes = new ArrayList<Node>();

		// delete old nodes
		for (final ONNode objectNode : new ArrayList<ONNode>(
				objectNet.getNodes())) {
			final Node aggNode = objectNode2graphNodeMap.get(objectNode);
			Assert.isNotNull(aggNode, "Incomplete node map!");
			if (aggNode.getContext() != null) {
				preservedNodes.add(aggNode);
				continue;
			}
			for (final ONArc onArc : new ArrayList<ONArc>(
					objectNode.getOutgoing())) {
				onArc.setObjectNet(null);
				onArc.setSource(null);
				onArc.setTarget(null);
			}
			for (final ONArc onArc : new ArrayList<ONArc>(
					objectNode.getIncoming())) {
				onArc.setObjectNet(null);
				onArc.setSource(null);
				onArc.setTarget(null);
			}
			objectNode.setObjectNet(null);
		}

		final HashMap<ONNode, Node> rhsNodeMap = objectNet2NodeMap.get(ronRule
				.getRhs());
		// create new nodes
		for (final Node aggNode : graph.getNodesList()) {
			if (!preservedNodes.contains(aggNode)) {
				// create place or transition node
				ONNode onNode = null;
				if (aggNode.getType().getStringRepr().equals("Place")) {
					onNode = RonmodelFactory.eINSTANCE.createONPlace();
					int token = 0;
					for (final Arc aggArc : aggNode.getIncomingArcsVec()) {
						if (aggArc.getSource().getType().getStringRepr()
								.equals("Token")) {
							token++;
						}
					}
					((ONPlace) onNode).setTokens(token);
				} else if (aggNode.getType().getStringRepr()
						.startsWith("Transition")) {
					onNode = RonmodelFactory.eINSTANCE.createONTransition();
				}

				// set attributes
				if (onNode != null) {
					onNode.setObjectNet(objectNet);
					onNode.setName((String) aggNode.getAttribute().getValueAt(
							"name"));
					final Enumeration<GraphObject> en = coMatch
							.getInverseImage(aggNode);
					while (en.hasMoreElements()) {
						final Node rhsNode = (Node) en.nextElement();
						final ONNode onRhsNode = inverseLookup(rhsNodeMap,
								rhsNode);
						Assert.isNotNull(onRhsNode,
								"Incomplete node map for RHS!");
						onNode.setX(onRhsNode.getX());
						onNode.setY(onRhsNode.getY());
					}
					objectNode2graphNodeMap.put(onNode, aggNode);
				}
			}
		}

		final HashMap<ONArc, Arc> rhsArcMap = objectNet2ArcMap.get(ronRule
				.getRhs());
		nextArc: for (final Arc aggArc : graph.getArcsList()) {
			if (aggArc.getType() == tokenArcType) {
				continue nextArc;
			}
			final Node aggSource = (Node) aggArc.getSource();
			final Node aggTarget = (Node) aggArc.getTarget();

			final ONNode onSource = inverseLookup(objectNode2graphNodeMap,
					aggSource);
			final ONNode onTarget = inverseLookup(objectNode2graphNodeMap,
					aggTarget);

			// check if source node is already connected with target node
			for (final ONArc onArc : onSource.getOutgoing()) {
				if (onArc.getTarget() == onTarget) {
					continue nextArc;
				}
			}
			// if not connected: create an arc
			final ONArc onArc = RonmodelFactory.eINSTANCE.createONArc();
			final Enumeration<GraphObject> en = coMatch.getInverseImage(aggArc);
			while (en.hasMoreElements()) {
				final ONArc onRhsArc = inverseLookup(rhsArcMap,
						(Arc) en.nextElement());
				Assert.isNotNull(onRhsArc, "Incomplete node map for RHS!");
				onArc.setWeight(onRhsArc.getWeight());
			}
			onArc.setSource(onSource);
			onArc.setTarget(onTarget);
			onArc.setObjectNet(objectNet);

		}
		return true;
	}

	public ArrayList<NetMorphism> getMatches(final Rule rule,
			final ObjectNet net) {
		if (!RuleChecker.isRuleOk(rule)) {
			return null;
		}
		final ArrayList<NetMorphism> matches = completeMatches(null, rule, net);
		// aggGrammar.save("C://Eclipse//test.ggx");
		return matches;
	}

	public void cleanGrammarCache() {
		objectNet2graph.clear();
		objectNet2NodeMap.clear();
		ronRule2aggRule.clear();
	}

	/**
	 * Note that the mappings in the result matches have no object net set!
	 */
	public ArrayList<NetMorphism> completeMatches(
			final NetMorphism netMorphism, final Rule onRule,
			final ObjectNet net) {

		Assert.isTrue(netMorphism == null || netMorphism.getDomain() == net
				&& netMorphism.getCodomain() == onRule.getLhs(),
				"Wrong NetMorphism type!");

		final ArrayList<NetMorphism> matches = new ArrayList<NetMorphism>();

		final GraTra trafo = new DefaultGraTraImpl();
		trafo.setGraGra(aggGrammar);
		trafo.setCompletionStrategy(aggGrammar.getMorphismCompletionStrategy());

		final agg.xt_basis.Rule aggRule = getAGGRule(onRule);
		final Graph graph = getAGGGraph(net);
		aggGrammar.resetGraph(graph);

		final Match aggMatch = trafo.createMatch(aggRule);
		mapping2Morphism(netMorphism, aggMatch);

		final Graph sourceGraph = aggMatch.getSource();
		final Graph targetGraph = aggMatch.getTarget();
		final ObjectNet sourceNet = inverseLookup(objectNet2graph, sourceGraph);
		final ObjectNet targetNet = inverseLookup(objectNet2graph, targetGraph);
		if (sourceNet != null && targetNet != null) {
			final HashMap<ONNode, Node> sourceNodeMap = objectNet2NodeMap
					.get(sourceNet);
			final HashMap<ONNode, Node> targetNodeMap = objectNet2NodeMap
					.get(targetNet);

			while (aggMatch.nextCompletion()) {
				final NetMorphism match = new NetMorphism();
				/*****************************************
				 * This was morphism2mapping
				 */
				final Enumeration<GraphObject> en = aggMatch.getDomain();
				while (en.hasMoreElements()) {
					final GraphObject sourceObject = en.nextElement();
					// FIXED: build match only for nodes that are not tokens
					if (sourceObject.isNode()
							&& sourceObject.getType() != tokenType) {
						final Node targetAggNode = (Node) aggMatch
								.getImage(sourceObject);

						final ONNode sourceONNode = inverseLookup(
								sourceNodeMap, (Node) sourceObject);
						final ONNode targetONNode = inverseLookup(
								targetNodeMap, targetAggNode);

						final Mapping mapping = RonmodelFactory.eINSTANCE
								.createMapping();
						mapping.setSource(sourceONNode);
						mapping.setTarget(targetONNode);
						match.add(mapping);
					}
				}
				matches.add(match);
			}
		}

		// aggGrammar.save("C://Eclipse//test.ggx");
		return matches;
	}

	public ArrayList<ArrayList<NetMorphism>> performCriticalPairAnalysis(
			final ObjectNet objectNet, final Rule rule1, final Rule rule2) {
		final ArrayList<ArrayList<NetMorphism>> result = new ArrayList<ArrayList<NetMorphism>>();

		final agg.xt_basis.Rule aggRule1 = getAGGRule(rule1);
		final agg.xt_basis.Rule aggRule2 = getAGGRule(rule2);
		final Graph graph = getAGGGraph(objectNet);

		// aggGrammar.save("C://test.ggx");

		final ExcludePairContainer epc = new ExcludePairContainer(aggGrammar);
		epc.enableUseHostGraph(true, graph);

		Vector<Pair<Pair<OrdinaryMorphism, OrdinaryMorphism>, Pair<OrdinaryMorphism, OrdinaryMorphism>>> criticalPairs = new Vector<Pair<Pair<OrdinaryMorphism, OrdinaryMorphism>, Pair<OrdinaryMorphism, OrdinaryMorphism>>>();
		Hashtable<Graph, Vector<Hashtable<GraphObject, GraphObject>>> resultOnHostGraph = new Hashtable<Graph, Vector<Hashtable<GraphObject, GraphObject>>>();

		try {
			criticalPairs = epc.getCriticalPair(aggRule1, aggRule2,
					CriticalPair.CONFLICT);
			resultOnHostGraph = epc.getCriticalForGraph(aggRule1, aggRule2);
		} catch (final Exception e) {
			System.out.println(e);
		}

		if (criticalPairs != null) {
			for (final Pair<Pair<OrdinaryMorphism, OrdinaryMorphism>, Pair<OrdinaryMorphism, OrdinaryMorphism>> overlappingPair : criticalPairs) {
				final Pair<OrdinaryMorphism, OrdinaryMorphism> aggCriticalPair = overlappingPair.first;
				final OrdinaryMorphism rule1Morphism = aggCriticalPair.first;
				final OrdinaryMorphism rule2Morphism = aggCriticalPair.second;
				final Graph overlappingGraph = rule1Morphism.getImage();
				final Vector<Hashtable<GraphObject, GraphObject>> matches = resultOnHostGraph
						.get(overlappingGraph);

				for (final Hashtable<GraphObject, GraphObject> morphism : matches) {
					final ArrayList<NetMorphism> ronCriticalPair = new ArrayList<NetMorphism>(
							2);
					final NetMorphism ronRule1Mapping = new NetMorphism();
					final NetMorphism ronRule2Mapping = new NetMorphism();

					final HashMap<ONNode, Node> rule1NodeMap = objectNet2NodeMap
							.get(inverseLookup(objectNet2graph,
									rule1Morphism.getSource()));
					final HashMap<ONNode, Node> rule2NodeMap = objectNet2NodeMap
							.get(inverseLookup(objectNet2graph,
									rule2Morphism.getSource()));
					final HashMap<ONNode, Node> objectNetNodeMap = objectNet2NodeMap
							.get(objectNet);

					final Enumeration<GraphObject> rule1Domain = rule1Morphism
							.getDomain();
					while (rule1Domain.hasMoreElements()) {
						final Mapping nodeMapping = RonmodelFactory.eINSTANCE
								.createMapping();
						final GraphObject aggSource = rule1Domain.nextElement();
						final GraphObject temp = rule1Morphism
								.getImage(aggSource);
						final GraphObject aggTarget = morphism.get(temp);

						if (aggSource instanceof Node && aggTarget != null) {
							final ONNode ronSource = inverseLookup(
									rule1NodeMap, (Node) aggSource);
							final ONNode ronTarget = inverseLookup(
									objectNetNodeMap, (Node) aggTarget);
							nodeMapping.setSource(ronSource);
							nodeMapping.setTarget(ronTarget);
							ronRule1Mapping.add(nodeMapping);
						}
					}

					final Enumeration<GraphObject> rule2Domain = rule2Morphism
							.getDomain();
					while (rule2Domain.hasMoreElements()) {
						final Mapping nodeMapping = RonmodelFactory.eINSTANCE
								.createMapping();
						final GraphObject aggSource = rule2Domain.nextElement();
						final GraphObject temp = rule2Morphism
								.getImage(aggSource);
						final GraphObject aggTarget = morphism.get(temp);

						if (aggSource instanceof Node && aggTarget != null) {
							final ONNode ronSource = inverseLookup(
									rule2NodeMap, (Node) aggSource);
							final ONNode ronTarget = inverseLookup(
									objectNetNodeMap, (Node) aggTarget);
							nodeMapping.setSource(ronSource);
							nodeMapping.setTarget(ronTarget);
							ronRule2Mapping.add(nodeMapping);
						}
					}
					ronCriticalPair.add(ronRule1Mapping);
					ronCriticalPair.add(ronRule2Mapping);
					result.add(ronCriticalPair);
				}
			}
		}

		return result;
	}

	private agg.xt_basis.Rule getAGGRule(final Rule ronRule) {
		agg.xt_basis.Rule aggRule = ronRule2aggRule.get(ronRule);
		if (aggRule != null) {
			return aggRule;
		}

		aggRule = aggGrammar.createRule();
		aggRule.setName(ronRule.getName());
		ronRule2aggRule.put(ronRule, aggRule);

		// convert LHS
		final LHS onLHS = ronRule.getLhs();
		final HashMap<ONNode, Node> lhsNodeMap = net2graph(onLHS,
				aggRule.getSource());

		// convert RHS and set node mapping
		final RHS onRHS = ronRule.getRhs();
		final HashMap<ONNode, Node> codomainNodeMap = net2graph(onRHS,
				aggRule.getTarget());

		for (final Mapping mapping : onRHS.getMappings()) {
			final GraphObject source = lhsNodeMap.get(mapping.getSource());
			final GraphObject target = codomainNodeMap.get(mapping.getTarget());
			aggRule.addMapping(source, target);
		}
		// set arc mapping
		aggRule.nextCompletion(aggRule.getSource().getNodes(), aggRule
				.getSource().getArcs());

		// convert NACs and set mapping
		for (final NAC onNAC : ronRule.getNacs()) {
			final OrdinaryMorphism nacMorphism = aggRule.createNAC();
			final HashMap<ONNode, Node> nacNodeMap = net2graph(onNAC,
					nacMorphism.getTarget());

			for (final Mapping mapping : onNAC.getMappings()) {
				final GraphObject source = lhsNodeMap.get(mapping.getSource());
				final GraphObject target = nacNodeMap.get(mapping.getTarget());
				nacMorphism.addMapping(source, target);
			}
			nacMorphism.nextCompletion(nacMorphism.getSource().getNodes(),
					nacMorphism.getSource().getArcs());
		}
		return aggRule;
	}

	private Graph getAGGGraph(final ObjectNet objectNet) {
		Graph graph = objectNet2graph.get(objectNet);
		if (graph != null) {
			return graph;
		}
		graph = new Graph(typeSet);
		aggGrammar.addGraph(graph);

		net2graph(objectNet, graph);

		return graph;
	}

	/**
	 * This method will convert an object net instance to an AGG graph.
	 * 
	 * @param objectNet
	 *            The object net which should be converted.
	 * @param graph
	 *            A preferably empty graph where the result will be stored.
	 */
	private HashMap<ONNode, Node> net2graph(final ObjectNet objectNet,
			final Graph graph) {
		final HashMap<ONNode, Node> net2graphNodeMap = new HashMap<ONNode, Node>();
		final HashMap<ONArc, Arc> net2graphArcMap = new HashMap<ONArc, Arc>();
		try {
			for (final ONNode onNode : objectNet.getNodes()) {
				Node aggNode;
				if (onNode instanceof ONPlace) {
					aggNode = graph.createNode(placeType);
					net2graphNodeMap.put(onNode, aggNode);
					for (int i = 0; i < ((ONPlace) onNode).getTokens(); i++) {
						final Node tokenNode = graph.createNode(tokenType);
						// tokenNode.getAttribute().setExprAt(Integer.valueOf(i),
						// "number");
						tokenNode.getAttribute().setValueAt(Integer.valueOf(i),
								"number");
						graph.createArc(tokenArcType, tokenNode, aggNode);
					}
				} else {
					// if (onNode instanceof ONTransition)
					final String transitionTypeName = "Transition_"
							+ onNode.getIncoming().size() + "_"
							+ onNode.getOutgoing().size();
					Type transitionType = typeSet
							.getTypeByName(transitionTypeName);
					if (transitionType == null) {
						transitionType = typeSet.createType(true);
						transitionType.setStringRepr(transitionTypeName);
						transitionType
								.setAdditionalRepr(":RECT:java.awt.Color[r=0,g=0,b=0]::[NODE]:");
						final AttrType attr = transitionType.getAttrType();
						attr.addMember(new JexHandler(), "String", "name");
					}
					aggNode = graph.createNode(transitionType);
					net2graphNodeMap.put(onNode, aggNode);
				}

				// set the node name
				if (onNode.getName() != null
						&& !onNode.getName().trim().isEmpty()) {
					aggNode.getAttribute().setExprValueAt(
							"\"" + onNode.getName().trim() + "\"", "name");
				}
			}

			for (final ONArc onArc : objectNet.getArcs()) {
				final ONNode onSource = onArc.getSource();
				final ONNode onTarget = onArc.getTarget();
				final Node aggSource = net2graphNodeMap.get(onSource);
				final Node aggTarget = net2graphNodeMap.get(onTarget);
				final Arc aggArc = graph.createArc(arcType, aggSource,
						aggTarget);
				aggArc.getAttribute().setExprAt(
						String.valueOf(onArc.getWeight()), "weight");
				net2graphArcMap.put(onArc, aggArc);
			}

			objectNet2NodeMap.put(objectNet, net2graphNodeMap);
			objectNet2ArcMap.put(objectNet, net2graphArcMap);
			objectNet2graph.put(objectNet, graph);
		} catch (final TypeException e) {
		}
		return net2graphNodeMap;
	}

	void mapping2Morphism(final NetMorphism mappings,
			final OrdinaryMorphism morphism) {
		if (mappings == null || morphism == null || mappings.isEmpty()) {
			return;
		}

		final ObjectNet sourceNet = mappings.getDomain();
		final ObjectNet targetNet = mappings.getCodomain();

		if (objectNet2graph.get(sourceNet) != morphism.getSource()
				|| objectNet2graph.get(targetNet) != morphism.getTarget()) {
			return;
		}

		final HashMap<ONNode, Node> sourceNodeMap = objectNet2NodeMap
				.get(sourceNet);
		final HashMap<ONNode, Node> targetNodeMap = objectNet2NodeMap
				.get(targetNet);

		for (final Mapping mapping : mappings) {
			final Node sourceAggNode = sourceNodeMap.get(mapping.getSource());
			final Node targetAggNode = targetNodeMap.get(mapping.getTarget());
			morphism.addMapping(sourceAggNode, targetAggNode);
		}
	}

}
