package aggSim.analyzer;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import muvitorkit.ui.utils.test.EditorJob;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.IJobChangeEvent;
import org.eclipse.core.runtime.jobs.JobChangeAdapter;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;

import roneditor.model.HLTransitionUtil;
import ronmodel.HLArc;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.HLTransition;
import ronmodel.HLTransitionKind;
import ronmodel.ObjectNet;
import ronmodel.Rule;
import ronmodel.Token;
import aggSim.Grammar;
import aggSim.analyzer.dialogs.LocalPlaceConflictAnalysisDialog;

/**
 * {@link SelectionAction} for performing an conflict analysis on either an
 * {@link ObjectNet} (Transition/Transition conflict), an {@link ObjectNet} and
 * a {@link Rule} (Transition/Rule application conflict) or an {@link ObjectNet}
 * and two {@link Rule}s (Rule application/Rule application conflict). The
 * result of the analysis is shown in an appropriate dialog.
 * 
 * Analysis is performed on copies of the actual models! So the dialogs showing
 * the results could not harm the working model.
 */
public class LocalPlaceConflictAnalysisAction extends SelectionAction {

	public static final String ID = "roneditor.actions.DirectConflictAnalysisAction";

	private static final String NAME = "Show local place conflicts";

	final ArrayList<HLTransition> loopApplyTransitions = new ArrayList<HLTransition>();

	boolean foundFireTransition;

	final HashMap<ObjectNet, List<Rule>> net2applicableRulesMap = new HashMap<ObjectNet, List<Rule>>();
	final HashMap<ObjectNet, ArrayList<Conflict>> net2transTransConflicts = new HashMap<ObjectNet, ArrayList<Conflict>>();
	final HashMap<ObjectNet, HashMap<Rule, List<Conflict>>> net2ruleTransConflictsMap = new HashMap<ObjectNet, HashMap<Rule, List<Conflict>>>();
	final HashMap<ObjectNet, HashMap<Rule, HashMap<Rule, List<Conflict>>>> net2ruleRuleConflictsMap = new HashMap<ObjectNet, HashMap<Rule, HashMap<Rule, List<Conflict>>>>();

	HLPlace selectedHLPlace;

	public LocalPlaceConflictAnalysisAction(final IWorkbenchPart part) {
		super(part);
		setId(ID);
		setText(NAME);
	}

	@Override
	public void run() {
		foundFireTransition = false;

		// compute intersection of incoming and outgoing transitions of
		// type FIRE or APPLYRULE
		for (final Object incoming : selectedHLPlace.getIncoming()) {
			final HLTransition preTrans = (HLTransition) ((HLArc) incoming)
					.getSource();
			// check for right transition type
			final int kind = preTrans.getKind().getValue();
			if (kind != HLTransitionKind.FIRE
					&& kind != HLTransitionKind.APPLYRULE) {
				continue;
			}
			// check if preTrans is also a postTrans and add it to the
			// list or remember that object nets on this HLPlace can be fired
			for (final HLArc outgoing : selectedHLPlace.getOutgoing()) {
				if (outgoing.getTarget() == preTrans) {
					if (kind == HLTransitionKind.FIRE) {
						foundFireTransition = true;
					} else {
						loopApplyTransitions.add(preTrans);
					}
					continue;
				}
			}
		}

		final EditorJob job = new EditorJob("Analyze conflicts") {
			private IProgressMonitor monitor;

			@Override
			public IStatus run(final IProgressMonitor monitor) {

				this.monitor = monitor;

				// count all rules that are involved and create working copies
				int totalWorkSteps = 0;
				final List<ObjectNet> objectNetCopies = new ArrayList<ObjectNet>();
				for (final Token token : selectedHLPlace.getTokens()) {
					final ObjectNet objectNet = (ObjectNet) token;
					/*
					 * collect all rules that could be applied to the actual
					 * object net by the looping transitions
					 */
					final ArrayList<Rule> applicableRules = getApplicableRuleCopies(objectNet);
					net2applicableRulesMap.put(objectNet, applicableRules);
					totalWorkSteps += 1 + applicableRules.size()
							+ Math.pow(applicableRules.size(), 2);
					objectNetCopies.add(objectNet);
				}

				// initialize progress monitor
				monitor.beginTask("Computing conflicts...", totalWorkSteps);

				// perform the analysis for each object net copy
				for (final ObjectNet objectNet : objectNetCopies) {
					net2transTransConflicts.put(objectNet,
							calculateTransitionConflicts(objectNet));
					/*
					 * Get Rule/Trans conflicts if a looping FIRE transition is
					 * connected to the host place
					 */
					net2ruleTransConflictsMap.put(objectNet,
							calculateRuleTransConflicts(objectNet));
					// Get Rule/Rule conflicts (stored in a curried mapping)
					net2ruleRuleConflictsMap.put(objectNet,
							calculateRuleRuleConflicts(objectNet));

				}
				monitor.done();
				return Status.OK_STATUS;
			}

			/**
			 * Get Trans/Trans conflicts if a looping FIRE transition is
			 * connected to the host place
			 */
			private ArrayList<Conflict> calculateTransitionConflicts(
					final ObjectNet objectNet) {
				final ArrayList<Conflict> conflicts = new ArrayList<Conflict>();
				monitor.setTaskName(objectNet.getName()
						+ ": Computing transition/transition conflicts...");
				if (foundFireTransition) {
					conflicts.addAll(Analyzer.analyzeTokenFiring(objectNet));
				}
				monitor.worked(1);
				return conflicts;
			}

			/**
			 * @param monitor
			 * @param rules
			 * @param ruleTransConflictsMap
			 * @throws InterruptedException
			 */
			private HashMap<Rule, List<Conflict>> calculateRuleTransConflicts(
					final ObjectNet objectNet) {
				final HashMap<Rule, List<Conflict>> conflicts = new HashMap<Rule, List<Conflict>>();
				int counter = 1;
				final List<Rule> rules = net2applicableRulesMap.get(objectNet);
				if (foundFireTransition) {
					for (final Rule rule : rules) {
						monitor.setTaskName(objectNet.getName()
								+ ": Computing rule/transition conflicts... "
								+ counter++ + "/" + rules.size());
						conflicts.put(rule, Analyzer
								.analyzeFiringTransformation(rule, objectNet));
						monitor.worked(1);
					}
				}
				return conflicts;
			}

			private HashMap<Rule, HashMap<Rule, List<Conflict>>> calculateRuleRuleConflicts(
					final ObjectNet objectNet) {
				final HashMap<Rule, HashMap<Rule, List<Conflict>>> conflicts = new HashMap<Rule, HashMap<Rule, List<Conflict>>>();
				int counter = 1;
				final List<Rule> rules = net2applicableRulesMap.get(objectNet);
				final int combinations = (int) Math.pow(rules.size(), 2);
				for (final Rule rule1 : rules) {
					final HashMap<Rule, List<Conflict>> innerMap = new HashMap<Rule, List<Conflict>>();
					// conflicts of a rule with itself are computed!
					for (final Rule rule2 : rules) {
						monitor.setTaskName(objectNet.getName()
								+ ": Computing rule/rule conflicts... "
								+ counter++ + "/" + combinations);
						innerMap.put(rule2, Analyzer.analyzeNetTransformation(
								objectNet, rule1, rule2));
						monitor.worked(1);
					}
					if (!innerMap.isEmpty()) {
						conflicts.put(rule1, innerMap);
					}
				}
				return conflicts;
			}
		};

		job.setUser(true);
		job.addJobChangeListener(new JobChangeAdapter() {
			@Override
			public void done(final IJobChangeEvent event) {
				if (event.getResult() != null) {
					// open dialog in UI thread
					final Display workbenchDisplay = PlatformUI.getWorkbench()
							.getDisplay();
					workbenchDisplay.asyncExec(new Runnable() {
						@Override
						public void run() {
							new LocalPlaceConflictAnalysisDialog(
									selectedHLPlace.getName(),
									net2transTransConflicts,
									net2ruleTransConflictsMap,
									net2ruleRuleConflictsMap);

							loopApplyTransitions.clear();
							net2transTransConflicts.clear();
							net2ruleTransConflictsMap.clear();
							net2ruleRuleConflictsMap.clear();
							net2applicableRulesMap.clear();
						}
					});
				}
			}

		});
		job.schedule();
	}

	/**
	 * This {@link SelectionAction} is enabled if exactly one {@link HLPlace} of
	 * type NET is selected.
	 */
	@Override
	protected boolean calculateEnabled() {
		selectedHLPlace = null;
		final IStructuredSelection selection = (IStructuredSelection) getSelection();
		if (selection.size() != 1) {
			return false;
		}
		final Object selectedElement = selection.getFirstElement();
		if (selectedElement instanceof EditPart) {
			final Object selectedModel = ((EditPart) selectedElement)
					.getModel();
			if (selectedModel instanceof HLPlace) {
				final HLPlace hlPlace = (HLPlace) selectedModel;
				if (hlPlace.getType() == HLPlaceType.NET_LITERAL) {
					selectedHLPlace = hlPlace;
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * @return
	 */
	ArrayList<Rule> getApplicableRuleCopies(final ObjectNet objectNet) {
		final ArrayList<Rule> rules = new ArrayList<Rule>();
		final Grammar grammar = new Grammar();
		for (final HLTransition trans : loopApplyTransitions) {
			for (final HLPlace rulePlace : HLTransitionUtil
					.getIncomingRulePlaces(trans)) {
				for (final Token token : rulePlace.getTokens()) {
					Assert.isTrue(token instanceof Rule);
					if (grammar.isApplicable((Rule) token, objectNet)) {
						rules.add((Rule) token);
					}
				}
			}
		}
		return rules;
	}
}
