package aggSim.analyzer.dialogs;

import java.util.ArrayList;
import java.util.List;

import muvitorkit.actions.GenericGraphLayoutAction;
import muvitorkit.actions.TrimViewerAction;
import muvitorkit.ui.utils.SWTResourceManager;

import org.eclipse.draw2d.ConnectionLayer;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.editparts.SimpleRootEditPart;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import roneditor.model.ONTransitionUtil;
import ronmodel.ONTransition;
import aggSim.analyzer.Conflict;

/**
 * This is an abstract implementation of a dialog to show {@link Conflict}s in
 * RONs. It provides a conflict counting and changing mechanism and helps to
 * create basic graphical viewers without any editing abilities.
 * 
 * @author "Tony Modica"
 */
public abstract class AbstractConflictDialog extends Dialog {

	private final Shell dialogShell;

	private final List<Conflict> conflicts;

	private final List<Conflict> filteredConflicts;

	private final EditPartFactory editPartFactory;

	protected int currentConflict = 0;

	private Button buttonPrevious;

	private Label labelCounter;

	private Button buttonNext;

	private Button buttonFilter;

	private boolean doFilter = false;

	private boolean doLayout = false;

	public AbstractConflictDialog(final List<Conflict> conflicts) {
		// seems to work, though I don't know if this is preferable this way
		super(Display.getDefault().getActiveShell() == null ? new Shell(
				Display.getDefault()) : Display.getDefault().getActiveShell());
		this.conflicts = conflicts;
		filteredConflicts = computeFilteredConflicts();
		dialogShell = new Shell(getParent(), SWT.DIALOG_TRIM
				| SWT.APPLICATION_MODAL | SWT.RESIZE);
		// create a shared edit part factory
		editPartFactory = createEditPartFactory();
	}

	/**
	 * Subclasses must implement this to provide a EditPartFactory that will be
	 * shared by the {@link GraphicalViewer}s created by
	 * {@link #createGraphicalViewer(Composite)}
	 * 
	 * @return an appropriate edit part factory for the graphical viewers to
	 *         display the conflicting models
	 */
	abstract protected EditPartFactory createEditPartFactory();

	/**
	 * Builds the dialog by creating the widget hierarchy and calls
	 * {@link #createControl()}, {@link #createControl()}, {@link #init()}, and
	 * {@link #optionalLayout()} in this order.
	 * 
	 * Concrete classes must implement {@link #createControl()} that does the
	 * work. open() just configures the {@link #dialogShell} and provides the
	 * usual control buttons on the bottom.
	 * 
	 * @see #createControl()
	 * @see #init()
	 */
	final public void open() {
		// Register as a resource user - SWTResourceManager will
		// handle the obtaining and disposing of resources
		SWTResourceManager.registerResourceUser(dialogShell);

		final GridLayout dialogShellLayout = new GridLayout();
		dialogShellLayout.makeColumnsEqualWidth = true;
		dialogShell.setLayout(dialogShellLayout);

		// create the main control parts
		createControl();

		// add the standard buttons
		createButtons();

		// set up the rest
		init();

		// initialize current conflict counter and buttons
		setCurrentConflict(0);
		if (getConflicts().size() < 2) {
			buttonNext.setEnabled(false);
			buttonPrevious.setEnabled(false);
		}

		dialogShell.open();

		// perform optional (edit part and figure) layout in viewers
		optionalLayout();

		// minimize the dialogs size
		dialogShell.layout();
		dialogShell.pack();

		// go into event loop
		final Display display = getDialogShell().getDisplay();
		while (!getDialogShell().isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}

		// if something has to be disposed (like Colors) do it here
		optionalDispose();
	}

	/**
	 * Subclasses must implement this to enable {@link #open()} to create the
	 * control with {@link #getDialogShell()} as top parent.
	 */
	abstract protected void createControl();

	/**
	 * Subclasses must implement this to create viewers on {@link Control}s
	 * (possibly {@link SashForm}s) and set their contents.
	 * 
	 * @see #createGraphicalViewer(Composite)
	 */
	abstract protected void init();

	/**
	 * Layout the object net automatically. This has to happen here so that the
	 * figures have bounds.
	 */
	protected void optionalLayout() {
		if (doLayout) {
			final GenericGraphLayoutAction layoutAction = new GenericGraphLayoutAction(
					null);
			layoutAction.setViewer(getMainViewer());
			layoutAction.run();

			// call synchronous figure update so that trim gets the new bounds
			((SimpleRootEditPart) getMainViewer().getRootEditPart())
					.getFigure().getUpdateManager().performUpdate();
		}

		final TrimViewerAction trimAction = new TrimViewerAction(null);
		trimAction.setViewer(getMainViewer());
		trimAction.run();
	}

	abstract protected GraphicalViewer getMainViewer();

	/**
	 * Subclasses may use this to dispose stuff when this dialog is going to be
	 * closed. This is also the best place to restore the objects of the
	 * conflict analysis to their initial state (unapply each conflict, reset
	 * used colors etc.)
	 */
	abstract protected void optionalDispose();

	/**
	 * Creates a basic graphical viewer with no editing capabilities.
	 * 
	 * @param parent
	 *            the parent composite to create the graphical viewer on
	 * @return A newly created and configured {@link ScrollingGraphicalViewer}
	 *         on the passed parent {@link Composite}.
	 * 
	 * @see #createEditPartFactory()
	 * @see #createGraphicalViewer(Composite, Color)
	 */
	protected GraphicalViewer createGraphicalViewer(final Composite parent,
			final EObject contents) {
		return createGraphicalViewer(parent, contents, null);
	}

	/**
	 * Creates a basic graphical viewer with no editing capabilities. This may
	 * be extended for special needs on the created graphical viewer.
	 * 
	 * @param parent
	 *            the parent composite to create the graphical viewer on
	 * @param color
	 *            The color the root editpart's figure should be set to
	 * @return A newly created and configured {@link ScrollingGraphicalViewer}
	 *         on the passed parent {@link Composite}.
	 * 
	 * @see #createEditPartFactory()
	 */
	protected GraphicalViewer createGraphicalViewer(final Composite parent,
			final EObject contents, final Color color) {
		// set the new viewer on the viewer composite
		final GraphicalViewer viewer = new ScrollingGraphicalViewer();
		viewer.createControl(parent);
		final ScalableFreeformRootEditPart rootEditPart = new ScalableFreeformRootEditPart();
		viewer.setRootEditPart(rootEditPart);
		viewer.setEditPartFactory(editPartFactory);
		// set antialias on connection layer, this may be turned off if
		// causing trouble
		((ConnectionLayer) rootEditPart
				.getLayer(LayerConstants.CONNECTION_LAYER))
				.setAntialias(SWT.ON);
		// some optional coloring
		final Display display = PlatformUI.getWorkbench().getDisplay();
		viewer.getControl().setBackground(
				display.getSystemColor(SWT.COLOR_LIST_BACKGROUND));
		viewer.getControl().setForeground(
				display.getSystemColor(SWT.COLOR_LIST_FOREGROUND));
		viewer.setContents(contents);
		if (color != null) {
			((GraphicalEditPart) viewer.getContents()).getFigure()
					.setBackgroundColor(color);
		}
		return viewer;
	}

	private void createButtons() {
		// START >> compositeButtonsLower
		final Composite compositeButtonsLower = new Composite(getDialogShell(),
				SWT.NONE);
		final RowLayout compositeButtonsLowerLayout = new RowLayout(
				org.eclipse.swt.SWT.HORIZONTAL);
		compositeButtonsLower.setLayout(compositeButtonsLowerLayout);
		final GridData compositeButtonsLowerLData = new GridData();
		compositeButtonsLowerLData.verticalAlignment = GridData.FILL;
		compositeButtonsLowerLData.horizontalAlignment = GridData.CENTER;
		compositeButtonsLower.setLayoutData(compositeButtonsLowerLData);

		{
			// we put the counter label on a composite to center it vertically
			// on the height of the buttons
			final Composite compositeFilter = new Composite(
					compositeButtonsLower, SWT.NONE);
			final GridLayout compositeFilterLayout = new GridLayout();
			compositeFilterLayout.makeColumnsEqualWidth = true;
			compositeFilter.setLayout(compositeFilterLayout);
			buttonFilter = new Button(compositeFilter, SWT.CHECK | SWT.CENTER);
			buttonFilter.setText("Show actual conflicts only");
			buttonFilter
					.setToolTipText("When selected, only those conflicts with currently enabled transitions are shown.");
			buttonFilter.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseUp(final MouseEvent evt) {
					buttonFilterMouseUp(evt);
				}
			});
			// deactivate "show actual conflicts" button if we have only actual
			// conflicts
			if (conflicts.size() == filteredConflicts.size()) {
				buttonFilter.setSelection(true);
				buttonFilter.setEnabled(false);
			}
		}

		{
			buttonPrevious = new Button(compositeButtonsLower, SWT.PUSH
					| SWT.CENTER);
			buttonPrevious.setText("-");
			buttonPrevious.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseUp(final MouseEvent evt) {
					buttonPreviousMouseUp(evt);
				}
			});
		}
		{
			// we put the counter label on a composite to center it vertically
			// on the height of the buttons
			final Composite compositeLabel = new Composite(
					compositeButtonsLower, SWT.NONE);
			final GridLayout compositeLabelLayout = new GridLayout();
			compositeLabelLayout.makeColumnsEqualWidth = true;
			compositeLabel.setLayout(compositeLabelLayout);
			{
				labelCounter = new Label(compositeLabel, SWT.NONE);
				labelCounter.setText("None");
			}
		}
		{
			buttonNext = new Button(compositeButtonsLower, SWT.PUSH
					| SWT.CENTER);
			buttonNext.setText("+");
			buttonNext.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseUp(final MouseEvent evt) {
					buttonNextMouseUp(evt);
				}
			});
		}
		// START >> buttonDone
		{
			final Button buttonDone = new Button(compositeButtonsLower,
					SWT.PUSH | SWT.CENTER);
			buttonDone.setText("Done");
			buttonDone.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseUp(final MouseEvent evt) {
					buttonDoneMouseUp();
				}
			});
		}
		// END << buttonDone
		// END << compositeButtonsLower
	}

	final void buttonDoneMouseUp() {
		getDialogShell().close();
	}

	final void buttonPreviousMouseUp(final MouseEvent evt) {
		setCurrentConflict((currentConflict - 1 + getConflicts().size())
				% getConflicts().size());
	}

	final void buttonNextMouseUp(final MouseEvent evt) {
		setCurrentConflict((currentConflict + 1) % getConflicts().size());
	}

	final void buttonFilterMouseUp(final MouseEvent evt) {
		doFilter = buttonFilter.getSelection();
		if (getConflicts().size() > 0) {
			setCurrentConflict(0);
		} else {
			setCurrentConflict(-1);
		}
	}

	/**
	 * This method sets a new conflict according to the pressed button. It uses
	 * {@link #applyConflict(int)} which must be implemented by subclasses to do
	 * th actual work of removing the previous set conflict and set the i-th
	 * from {@link #getConflicts()}.
	 * 
	 * @param i
	 *            the position of the conflict to be set
	 */
	private void setCurrentConflict(final int i) {
		applyConflict(i);
		currentConflict = i;
		// change label inscription
		labelCounter.setText(i + 1 + "/" + getConflicts().size());
		labelCounter.getParent().getParent().layout();
		labelCounter.getParent().getParent().pack();
	}

	/**
	 * This must be implemented by subclasses to do the actual work of removing
	 * the previous set conflict and setting the i-th conflict from
	 * {@link #getConflicts()}. This method will be called by
	 * {@link #setCurrentConflict(int)} when appropriate and is not meant to be
	 * called in subclasses!
	 * 
	 * Implementors should be aware that when this method is called first no
	 * conflict has been set so far!
	 * 
	 * When the passed value is -1 then all conflicts have been filtered out. In
	 * this case this method should only "remove" the previous conflicts and
	 * return because {@link #getConflicts()} is empty.
	 * 
	 * @param i
	 *            the position of the conflict to be set, -1 when no (actual)
	 *            conflict exists
	 */
	abstract protected void applyConflict(int i);

	/**
	 * @return the dialogShell
	 */
	final protected Shell getDialogShell() {
		return dialogShell;
	}

	/**
	 * @return the conflicts
	 */
	final protected List<Conflict> getConflicts() {
		if (doFilter) {
			return filteredConflicts;
		}
		return conflicts;
	}

	private List<Conflict> computeFilteredConflicts() {
		final List<Conflict> filtered = new ArrayList<Conflict>();
		for (final Conflict conflict : conflicts) {
			// filter conflicts with inactive transitions
			final Object first = conflict.getFirst();
			final Object second = conflict.getSecond();
			if (first instanceof ONTransition
					&& !ONTransitionUtil.canFire((ONTransition) first)) {
				continue;
			}
			if (second instanceof ONTransition
					&& !ONTransitionUtil.canFire((ONTransition) second)) {
				continue;
			}
			filtered.add(conflict);
		}
		return filtered;
	}

	public void setDoLayout(final boolean doLayout) {
		this.doLayout = doLayout;
	}
}
