package muvitorkit.dialogs;

import java.util.ArrayList;
import java.util.List;

import muvitorkit.gef.palette.MappingCreationTool;
import muvitorkit.ui.ContextMenuProviderWithActionRegistry;
import muvitorkit.ui.utils.SWTResourceManager;

import org.eclipse.core.runtime.Assert;
import org.eclipse.draw2d.ConnectionLayer;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.Request;
import org.eclipse.gef.Tool;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.CreationFactory;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Layout;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import roneditor.gef.actions.ron.RunAutoSimulationAction;
import roneditor.model.RuleUtil;
import ronmodel.LHS;
import ronmodel.Mapping;
import ronmodel.ONNode;
import ronmodel.ObjectNet;
import ronmodel.RHS;
import ronmodel.Rule;
import aggSim.Grammar;
import aggSim.NetMorphism;

/**
 * @author "Tony Modica"
 */
abstract public class AbstractMatchDialog extends Dialog {

	/**
	 * A switch to determine whether this dialog has been canceled.
	 */
	private boolean canceled = true;

	/**
	 * The rules that can be selected to be applied in this dialog.
	 */
	private final ArrayList<Rule> rules;

	/**
	 * The object nets that can be selected to apply rules on in this dialog.
	 */
	private final ArrayList<ObjectNet> objectNets;

	/**
	 * The edit domain handling the mapping tool and the graphical viewers.
	 */
	private EditDomain editDomain;

	/**
	 * A viewer showing a copy of the current rule's {@link LHS}.
	 */
	private GraphicalViewer lhsViewer;

	/**
	 * A viewer showing a copy of the current rule's {@link RHS}.
	 */
	private GraphicalViewer rhsViewer;

	/**
	 * A viewer showing a copy of the current object net.
	 */
	private GraphicalViewer netViewer;

	/**
	 * A cache for the match between the current rule and object net that has
	 * been set in the viewers.
	 */
	private NetMorphism currentMatch;

	/**
	 * The rule of whose {@link LHS} a copy is being shown in the lhsViewer.
	 */
	private Rule currentRule;

	/**
	 * A copy of the currently selected object net.
	 */
	private ObjectNet currentObjectNet;

	/**
	 * If this is <code>null</code> we are in match completion mode! Otherwise
	 * it hold the (filtered) completions returned by AGG.
	 */
	private List<NetMorphism> matchCompletions;

	/**
	 * The position of the current match completion in the matchCompletions
	 * list. This is stored to be able to restore the right completion with undo
	 * and redo.
	 */
	private int currentCompletionPos;

	// Visual stuff
	private Shell dialogShell;
	private Text textMessage;
	private Button buttonAllCompletions;
	private Button buttonCancel;
	private Button buttonCalculateAndReset;
	private Button buttonApplyRule;
	private Composite compositeApplyLower;
	private Button buttonNextMatch;
	private Button buttonPreviousMatch;
	private Composite compositeMatchMiddle;
	private Button buttonNextNet;
	private Button buttonPreviousNet;
	private Composite compositeNetMiddle;
	private Composite compositeMiddle;
	private SashForm mainSashForm;
	private SashForm ruleSashForm;
	private Button buttonNextRule;
	private Button buttonPreviousRule;
	private Composite compositeRulesUpper;
	private EditPartFactory editPartFactory;

	private final Grammar grammar;

	/**
	 * Main constructor for this dialog. Note that you should pass copies only,
	 * because the result mappings ({@link #getMatchToApply()}) will be pointing
	 * to the input rules and object nets.
	 * 
	 * @param rules
	 *            The rules that can be switched through for match-setting. Must
	 *            contain at least 1 rule.
	 * @param objectNets
	 *            The target object nets that can be switched through for
	 *            match-setting. Must contain at least 1 object net.
	 */
	public AbstractMatchDialog(final ArrayList<Rule> rules,
			final ArrayList<ObjectNet> objectNets) {
		super(new Shell(Display.getDefault()), SWT.NULL);
		Assert.isTrue(rules != null && !rules.isEmpty(),
				"MatchDialog requires at least one rule!");
		Assert.isTrue(objectNets != null && !objectNets.isEmpty(),
				"MatchDialog requires at least one object net!");
		this.rules = rules;
		this.objectNets = objectNets;
		grammar = new Grammar();
	}

	/**
	 * Builds the dialog by creating the widget hierarchy and calling
	 * {@link #init()}.
	 * 
	 * @see #init()
	 */
	public void open() {
		try {
			final Shell parent = getParent();
			dialogShell = new Shell(parent, SWT.DIALOG_TRIM
					| SWT.APPLICATION_MODAL | SWT.RESIZE);

			{
				// Register as a resource user - SWTResourceManager will
				// handle the obtaining and disposing of resources
				SWTResourceManager.registerResourceUser(dialogShell);
			}

			final GridLayout dialogShellLayout = new GridLayout();
			dialogShell.setLayout(dialogShellLayout);

			dialogShell.setText("Specify match for rule application");
			// START >> compositeRulesUpper
			final GridData compositeUpperLData = new GridData();
			compositeUpperLData.grabExcessHorizontalSpace = true;
			compositeRulesUpper = new Composite(dialogShell, SWT.NONE);
			final GridLayout compositeUpperLayout = new GridLayout();
			compositeUpperLayout.numColumns = 2;
			compositeUpperLayout.makeColumnsEqualWidth = true;
			compositeRulesUpper.setLayout(compositeUpperLayout);
			compositeRulesUpper.setLayoutData(compositeUpperLData);
			// START >> buttonPreviousRule
			buttonPreviousRule = new Button(compositeRulesUpper, SWT.PUSH
					| SWT.CENTER);
			final GridData buttonPreviousRuleLData = new GridData();
			buttonPreviousRuleLData.horizontalAlignment = GridData.CENTER;
			buttonPreviousRule.setLayoutData(buttonPreviousRuleLData);
			buttonPreviousRule.setText("Previous Rule");
			buttonPreviousRule.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseUp(final MouseEvent evt) {
					buttonPreviousRuleMouseUp(evt);
				}
			});
			// END << buttonPreviousRule
			// START >> buttonNextRule
			buttonNextRule = new Button(compositeRulesUpper, SWT.PUSH
					| SWT.CENTER);
			final GridData buttonNextRuleLData = new GridData();
			buttonNextRuleLData.horizontalAlignment = GridData.CENTER;
			buttonNextRule.setLayoutData(buttonNextRuleLData);
			buttonNextRule.setText("Next Rule");
			buttonNextRule.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseUp(final MouseEvent evt) {
					buttonNextRuleMouseUp(evt);
				}
			});
			// END << buttonNextRule
			// END << compositeRulesUpper
			// START >> mainSashForm
			mainSashForm = new SashForm(dialogShell, SWT.BORDER | SWT.VERTICAL);
			final Layout mainSashFormLayout = new RowLayout(
					org.eclipse.swt.SWT.VERTICAL);
			mainSashForm.setLayout(mainSashFormLayout);
			mainSashForm.setBackground(getParent().getDisplay().getSystemColor(
					SWT.COLOR_LIST_BACKGROUND));
			final GridData mainSashFormLData = new GridData();
			mainSashFormLData.verticalAlignment = GridData.FILL;
			mainSashFormLData.horizontalAlignment = GridData.FILL;
			mainSashFormLData.grabExcessHorizontalSpace = true;
			mainSashFormLData.grabExcessVerticalSpace = true;
			mainSashForm.setLayoutData(mainSashFormLData);
			mainSashForm.setForeground(getParent().getDisplay().getSystemColor(
					SWT.COLOR_LIST_FOREGROUND));
			// END << mainSashForm
			// START >> ruleSashForm
			ruleSashForm = new SashForm(mainSashForm, SWT.BORDER
					| SWT.HORIZONTAL);
			final FillLayout ruleSashFormLayout = new FillLayout(
					org.eclipse.swt.SWT.VERTICAL);
			ruleSashForm.setLayout(ruleSashFormLayout);
			ruleSashForm.setBackground(getParent().getDisplay().getSystemColor(
					SWT.COLOR_LIST_BACKGROUND));
			final GridData ruleSashFormLData = new GridData();
			ruleSashFormLData.verticalAlignment = GridData.FILL;
			ruleSashFormLData.horizontalAlignment = GridData.FILL;
			ruleSashFormLData.grabExcessHorizontalSpace = true;
			ruleSashFormLData.grabExcessVerticalSpace = true;
			ruleSashForm.setLayoutData(ruleSashFormLData);
			ruleSashForm.setForeground(getParent().getDisplay().getSystemColor(
					SWT.COLOR_LIST_FOREGROUND));
			// END << ruleSashForm
			// START >> compositeMiddle
			final GridData compositeMiddleLData = new GridData();
			compositeMiddleLData.horizontalAlignment = GridData.FILL;
			compositeMiddleLData.grabExcessHorizontalSpace = true;
			compositeMiddle = new Composite(dialogShell, SWT.NONE);
			final GridLayout compositeMiddleLayout = new GridLayout();
			compositeMiddleLayout.numColumns = 2;
			compositeMiddle.setLayout(compositeMiddleLayout);
			compositeMiddle.setLayoutData(compositeMiddleLData);
			// START >> compositeNetMiddle
			final GridData compositeNetMiddleLData = new GridData();
			compositeNetMiddleLData.grabExcessHorizontalSpace = true;
			compositeNetMiddle = new Composite(compositeMiddle, SWT.NONE);
			final GridLayout compositeNetMiddleLayout = new GridLayout();
			compositeNetMiddleLayout.numColumns = 2;
			compositeNetMiddleLayout.makeColumnsEqualWidth = true;
			compositeNetMiddle.setLayout(compositeNetMiddleLayout);
			compositeNetMiddle.setLayoutData(compositeNetMiddleLData);
			// START >> buttonPreviousNet
			buttonPreviousNet = new Button(compositeNetMiddle, SWT.PUSH
					| SWT.CENTER);
			final GridData buttonPreviousNetLData = new GridData();
			buttonPreviousNetLData.horizontalAlignment = GridData.CENTER;
			buttonPreviousNet.setLayoutData(buttonPreviousNetLData);
			buttonPreviousNet.setText("Previous Object Net");
			buttonPreviousNet.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseUp(final MouseEvent evt) {
					buttonPreviousNetMouseUp(evt);
				}
			});
			// END << buttonPreviousNet
			// START >> buttonNextNet
			buttonNextNet = new Button(compositeNetMiddle, SWT.PUSH
					| SWT.CENTER);
			final GridData buttonNextNetLData = new GridData();
			buttonNextNetLData.horizontalAlignment = GridData.CENTER;
			buttonNextNet.setLayoutData(buttonNextNetLData);
			buttonNextNet.setText("Next Object Net");
			buttonNextNet.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseUp(final MouseEvent evt) {
					buttonNextNetMouseUp(evt);
				}
			});
			// END << buttonNextNet
			// END << compositeNetMiddle
			// START >> compositeMatchMiddle
			final GridData composite1LData = new GridData();
			composite1LData.horizontalAlignment = GridData.END;
			compositeMatchMiddle = new Composite(compositeMiddle, SWT.NONE);
			final GridLayout composite1Layout = new GridLayout();
			composite1Layout.numColumns = 3;
			compositeMatchMiddle.setLayout(composite1Layout);
			compositeMatchMiddle.setLayoutData(composite1LData);
			// START >> buttonPreviousMatch
			buttonPreviousMatch = new Button(compositeMatchMiddle, SWT.PUSH
					| SWT.CENTER);
			final GridData buttonPreviousMatchLData = new GridData();
			buttonPreviousMatchLData.horizontalAlignment = GridData.CENTER;
			buttonPreviousMatch.setLayoutData(buttonPreviousMatchLData);
			buttonPreviousMatch.setText("Previous completion");
			buttonPreviousMatch.setEnabled(false);
			buttonPreviousMatch
					.setToolTipText("Show previous match found by AGG");
			buttonPreviousMatch.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseUp(final MouseEvent evt) {
					buttonPreviousMatchMouseUp();
				}
			});
			// END << buttonPreviousMatch
			// START >> buttonNextMatch
			buttonNextMatch = new Button(compositeMatchMiddle, SWT.PUSH
					| SWT.CENTER);
			final GridData buttonFindMatchLData = new GridData();
			buttonFindMatchLData.horizontalAlignment = GridData.CENTER;
			buttonNextMatch.setLayoutData(buttonFindMatchLData);
			buttonNextMatch.setText("Next completion");
			buttonNextMatch.setEnabled(false);
			buttonNextMatch.setToolTipText("Show next match found by AGG");
			buttonNextMatch.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseUp(final MouseEvent evt) {
					buttonNextMatchMouseUp();
				}
			});
			// END << buttonNextMatch
			// END << compositeMatchMiddle
			// END << compositeMiddle
			// START >> compositeApplyLower
			final GridData compositeLData = new GridData();
			compositeLData.verticalAlignment = GridData.END;
			compositeLData.horizontalAlignment = GridData.CENTER;
			compositeLData.grabExcessHorizontalSpace = true;
			compositeApplyLower = new Composite(dialogShell, SWT.NONE);
			final GridLayout compositeLayout = new GridLayout();
			compositeLayout.numColumns = 4;
			compositeLayout.makeColumnsEqualWidth = true;
			compositeApplyLower.setLayout(compositeLayout);
			compositeApplyLower.setLayoutData(compositeLData);
			// START >> buttonApplyRule
			buttonApplyRule = new Button(compositeApplyLower, SWT.PUSH
					| SWT.CENTER);
			final GridData buttonApplyRuleLData = new GridData();
			buttonApplyRuleLData.horizontalAlignment = GridData.CENTER;
			buttonApplyRule.setLayoutData(buttonApplyRuleLData);
			buttonApplyRule.setText("Apply Rule");
			buttonApplyRule.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseUp(final MouseEvent evt) {
					buttonApplyRuleMouseUp();
				}
			});
			// END << buttonApplyRule
			// START >> buttonCancel
			buttonCancel = new Button(compositeApplyLower, SWT.PUSH
					| SWT.CENTER);
			final GridData buttonCancelLData = new GridData();
			buttonCancelLData.horizontalAlignment = GridData.CENTER;
			buttonCancel.setLayoutData(buttonCancelLData);
			buttonCancel.setText("Cancel");
			buttonCancel.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseUp(final MouseEvent evt) {
					buttonCancelMouseUp();
				}
			});
			// END << buttonCancel
			// START >> buttonCalculateAndReset
			buttonCalculateAndReset = new Button(compositeApplyLower, SWT.PUSH
					| SWT.CENTER);
			buttonCalculateAndReset.setEnabled(false);
			final GridData buttonCalculateAndResetLData = new GridData();
			buttonCalculateAndResetLData.horizontalAlignment = GridData.CENTER;
			buttonCalculateAndReset.setLayoutData(buttonCalculateAndResetLData);
			buttonCalculateAndReset.setText("Calculate Completions");
			buttonCalculateAndReset.setEnabled(true);
			buttonCalculateAndReset
					.setToolTipText("Calculate all match completions with AGG");
			buttonCalculateAndReset.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseUp(final MouseEvent evt) {
					buttonCalculateAndResetUp();
				}
			});
			// END << buttonCalculateAndReset
			// START >> buttonAllCompletions
			buttonAllCompletions = new Button(compositeApplyLower, SWT.CHECK
					| SWT.LEFT);
			final GridData buttonAllCompletionsLData = new GridData();
			buttonAllCompletionsLData.horizontalAlignment = GridData.CENTER;
			buttonAllCompletions.setEnabled(false);
			buttonAllCompletions.setLayoutData(buttonAllCompletionsLData);
			buttonAllCompletions.setText("All Completions");
			buttonAllCompletions
					.setToolTipText("Unset this if a partial match should be considered when computing match completions.");
			buttonAllCompletions.setSelection(true);
			// END << buttonAllCompletions
			// END << compositeApplyLower
			// START >> textMessage
			textMessage = new Text(dialogShell, SWT.MULTI | SWT.WRAP);
			final GridData textMessageLData = new GridData();
			textMessageLData.verticalAlignment = GridData.END;
			textMessageLData.horizontalAlignment = GridData.CENTER;
			textMessageLData.grabExcessHorizontalSpace = true;
			textMessage.setLayoutData(textMessageLData);
			textMessage.setEditable(false);
			textMessage.setBackground(SWTResourceManager
					.getColor(236, 233, 216));
			textMessage.setForeground(SWTResourceManager.getColor(255, 0, 0));
			// END << textMessage

			// set up the rest
			init();

			dialogShell.layout();
			dialogShell.pack();
			dialogShell.open();
			final Display display = dialogShell.getDisplay();
			while (!dialogShell.isDisposed()) {
				if (!display.readAndDispatch()) {
					display.sleep();
				}
			}
		} catch (final Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Initialize some fields, create graphical viewers and container rule, set
	 * first rule and object net, configure buttons' status and reset match
	 * completion related stuff.
	 */
	private void init() {
		// initialize some fields
		currentMatch = new NetMorphism();
		lhsViewer = createGraphicalViewer(ruleSashForm);
		rhsViewer = createGraphicalViewer(ruleSashForm);
		netViewer = createGraphicalViewer(mainSashForm);

		// set first rule and first net
		setCurrentNet(0);
		setCurrentRule(0);

		// allow switching only if multiple objects are available
		buttonNextRule.setEnabled(rules.size() > 1);
		buttonPreviousRule.setEnabled(rules.size() > 1);
		buttonNextNet.setEnabled(objectNets.size() > 1);
		buttonPreviousNet.setEnabled(objectNets.size() > 1);

		// initialize match completion buttons and mechanism
		// resetMatchCompletions();

		// we do not support partial matches yet... just offer all possible
		// matches
		buttonCalculateAndResetUp();
		buttonCalculateAndReset.setEnabled(false);
	}

	/**
	 * @return <code>true</code> if the match currently defined between the
	 *         actually shown {@link LHS} and {@link ObjectNet} is complete and
	 *         applicable.
	 * 
	 * @see #cacheCurrentMatch()
	 * @see #isMatchToApplyComplete()
	 */
	private boolean checkCurrentMatch() {
		if (isMatchToApplyComplete()) {
			return grammar.isApplicable(currentMatch, currentRule,
					currentObjectNet);
		}
		return false;
	}

	/**
	 * Helper method for {@link #checkCurrentMatch()}.
	 * 
	 * @return <code>true</code> if for each {@link ONNode} in the current
	 *         {@link LHS} a {@link Mapping} exists.
	 */
	private boolean isMatchToApplyComplete() {
		if (currentRule == null) {
			return false;
		}
		return currentMatch.containsAll(currentRule.getLhs().getNodes());
	}

	/**
	 * @return A newly created and configured {@link ScrollingGraphicalViewer}
	 *         on the passed parent composite.
	 * 
	 * @param parentComposite
	 * 
	 * @see #getEditDomain()
	 * @see #getEditPartFactory()
	 */
	private GraphicalViewer createGraphicalViewer(
			final Composite parentComposite) {
		// set the new viewer on the viewer composite
		final GraphicalViewer viewer = new ScrollingGraphicalViewer();
		viewer.createControl(parentComposite);
		viewer.setEditDomain(getEditDomain());
		getEditDomain().addViewer(viewer);
		final ScalableFreeformRootEditPart rootEditPart = new ScalableFreeformRootEditPart();
		viewer.setRootEditPart(rootEditPart);
		viewer.setEditPartFactory(getEditPartFactory());
		// viewer
		// .setContextMenu(createContextMenuProvider(viewer,
		// actionRegistry));
		// set antialias on connection layer, this may be turned off if
		// causing trouble
		((ConnectionLayer) rootEditPart
				.getLayer(LayerConstants.CONNECTION_LAYER))
				.setAntialias(SWT.ON);
		// some optional coloring
		viewer.getControl().setBackground(
				PlatformUI.getWorkbench().getDisplay()
						.getSystemColor(SWT.COLOR_LIST_BACKGROUND));
		viewer.getControl().setForeground(
				PlatformUI.getWorkbench().getDisplay()
						.getSystemColor(SWT.COLOR_LIST_FOREGROUND));
		return viewer;
	}

	/**
	 * @param viewer
	 * @param actionRegistry
	 * @return
	 */
	abstract protected ContextMenuProviderWithActionRegistry createContextMenuProvider(
			GraphicalViewer viewer);

	/**
	 * @return A lazily created {@link EditDomain} that works on this dialog's
	 *         command stack with a {@link Tool} for creating Mappings as
	 *         default tool.
	 * 
	 * @see #getCommandStack()
	 */
	protected EditDomain getEditDomain() {
		if (null == editDomain) {
			editDomain = new EditDomain();
			final MappingCreationTool mappingTool = new MappingCreationTool() {
				@Override
				protected Request createTargetRequest() {
					final CreateConnectionRequest request = (CreateConnectionRequest) super
							.createTargetRequest();
					request.getExtendedData().put("MatchDialogMode",
							Boolean.TRUE);
					return request;
				}
			};
			mappingTool.setFactory(createMappingCreationFactory());
			editDomain.setDefaultTool(mappingTool);
			editDomain.loadDefaultTool();
		}
		return editDomain;
	}

	/**
	 * @return
	 */
	abstract protected CreationFactory createMappingCreationFactory();

	/**
	 * @return An lazily created {@link EditPartFactory} for the
	 *         {@link GraphicalViewer}s created in
	 *         {@link #createGraphicalViewer()}.
	 */
	private EditPartFactory getEditPartFactory() {
		if (editPartFactory == null) {
			editPartFactory = createEditPartFactory();
		}
		return editPartFactory;
	}

	/**
	 * @return
	 */
	abstract protected EditPartFactory createEditPartFactory();

	/**
	 * @param rulePos
	 */
	private void setCurrentRule(final int rulePos) {
		if (currentObjectNet != null) {
			currentObjectNet.eAdapters().remove(this);
			// delete all mappings from old currentRule to currentNet
			deleteCurrentMatch();
		}

		// set new currentRule
		currentRule = rules.get(rulePos);
		currentRule.getLhs().setName(currentRule.getName());

		// RuleUtil.updateColors(currentRule);

		// update viewers
		lhsViewer.setContents(currentRule.getLhs());
		lhsViewer.getContents().refresh();
		rhsViewer.setContents(currentRule.getRhs());
		rhsViewer.getContents().refresh();

		dialogShell.layout();
		dialogShell.pack();

		// reset match completion mechanism
		resetMatchCompletions();
	}

	/**
	 * @param objectNet
	 */
	private void setCurrentNet(final int objectNetPos) {
		if (currentObjectNet != null) {
			currentObjectNet.eAdapters().remove(this);
			// delete all mappings from old currentRule to currentNet
			deleteCurrentMatch();
		}

		// set new currentObjectNet
		currentObjectNet = objectNets.get(objectNetPos);

		// TODO for partial match completion: need to listen to the
		// currentObjectNet for mapping changes!

		// update viewers
		netViewer.setContents(currentObjectNet);
		netViewer.getContents().refresh();

		dialogShell.layout();
		dialogShell.pack();

		resetMatchCompletions();
	}

	/**
	 * We delete all mappings contained in the {@link #currentObjectNet} with a
	 * source node in the {@link #currentRule}.
	 */
	private void deleteCurrentMatch() {
		for (final Mapping mapping : new ArrayList<Mapping>(
				currentObjectNet.getMappings())) {
			final ObjectNet sourceNet = mapping.getSource().getObjectNet();
			final ObjectNet targetNet = mapping.getTarget().getObjectNet();
			if (sourceNet == currentRule.getLhs()
					&& targetNet == currentObjectNet) {
				mapping.setCodomain(null);
				// mapping.setSource(null);
				// mapping.setTarget(null);
			}
		}
		RuleUtil.resetColors(currentObjectNet);
	}

	/**
	 * @see #init()
	 * @see #setCurrentNet(int)
	 * @see #setCurrentRule(int)
	 * @see #notifyChanged(Notification)
	 */
	private void resetMatchCompletions() {
		buttonNextMatch.setText("Next completion");
		buttonNextMatch.setEnabled(false);
		buttonPreviousMatch.setText("Previous completion");
		buttonPreviousMatch.setEnabled(false);
		deleteCurrentMatch();
		buttonCalculateAndReset.setText("Calculate completions");
		buttonCalculateAndReset
				.setToolTipText("Calculate all match completions with AGG");
		textMessage.setText("");

		// maybe the initial rule is applicable with an empty match?
		buttonApplyRule.setEnabled(checkCurrentMatch());

		compositeMatchMiddle.layout(true);
		compositeMatchMiddle.pack();
		// the following pointing to null signals that we are not in completion
		// mode
		matchCompletions = null;
		currentCompletionPos = -1;
	}

	/***************************************************************************
	 * Button handlers
	 **************************************************************************/

	/**
	 *
	 */
	void buttonPreviousMatchMouseUp() {
		setMatchCompletionAsCurrent(currentCompletionPos - 1);
	}

	/**
	 *
	 */
	void buttonNextMatchMouseUp() {
		setMatchCompletionAsCurrent(currentCompletionPos + 1);
	}

	private List<NetMorphism> computeMatchCompletions() {
		final List<NetMorphism> allCompletions = grammar.completeMatches(null,
				currentRule, currentObjectNet);

		// return matches if "All completions" is selected
		if (buttonAllCompletions.getSelection()) {
			return allCompletions;
		}

		// else apply conditional filtering: refuse completions that don't
		// include the current match
		final List<NetMorphism> filtered = new ArrayList<NetMorphism>();
		for (final NetMorphism completion : allCompletions) {
			// does completion contain current match?
			if (completion.contains(currentMatch)) {
				filtered.add(completion);
			}
		}
		return filtered;
	}

	/**
	 * @param newCompletionPos
	 */
	private void setMatchCompletionAsCurrent(final int newCompletionPos) {

		// delete current match
		deleteCurrentMatch();

		// set new current completion
		currentCompletionPos = newCompletionPos;
		currentMatch = matchCompletions.get(currentCompletionPos);

		// add new match completion
		for (final Mapping mapping : currentMatch) {
			mapping.setCodomain(currentObjectNet);
		}

		RuleUtil.updateColors(currentObjectNet);

		// as we have a completion, activate he apply button!
		buttonApplyRule.setEnabled(true);

		// update "previous completion" button
		// not first completion?
		if (currentCompletionPos > 0) {
			buttonPreviousMatch.setEnabled(true);
		} else {
			buttonPreviousMatch.setEnabled(false);
		}
		buttonPreviousMatch.setText("Previous completion" + "("
				+ currentCompletionPos + ")");
		// update "next completion" button
		// reached end of match list?
		if (currentCompletionPos < matchCompletions.size() - 1) {
			buttonNextMatch.setEnabled(true);
		} else {
			buttonNextMatch.setEnabled(false);
		}
		buttonNextMatch.setText("Next completion" + "("
				+ (matchCompletions.size() - 1 - currentCompletionPos) + ")");
		compositeMatchMiddle.layout(true);
		compositeMatchMiddle.pack();
	}

	/**
	 * @param evt
	 */
	void buttonPreviousRuleMouseUp(final MouseEvent evt) {
		final int currentRulePos = rules.indexOf(currentRule);
		if (currentRulePos == 0) {
			setCurrentRule(rules.size() - 1);
		} else {
			setCurrentRule(currentRulePos - 1);
		}
	}

	/**
	 * @param evt
	 */
	void buttonNextRuleMouseUp(final MouseEvent evt) {
		final int currentRulePos = rules.indexOf(currentRule);
		if (currentRulePos == rules.size() - 1) {
			setCurrentRule(0);
		} else {
			setCurrentRule(currentRulePos + 1);
		}
	}

	/**
	 * @param evt
	 */
	void buttonPreviousNetMouseUp(final MouseEvent evt) {
		final int currentObjectNetPos = objectNets.indexOf(currentObjectNet);
		if (currentObjectNetPos == 0) {
			setCurrentNet(objectNets.size() - 1);
		} else {
			setCurrentNet(currentObjectNetPos - 1);
		}
	}

	/**
	 * @param evt
	 */
	void buttonNextNetMouseUp(final MouseEvent evt) {
		final int currentObjectNetPos = objectNets.indexOf(currentObjectNet);
		if (currentObjectNetPos == objectNets.size() - 1) {
			setCurrentNet(0);
		} else {
			setCurrentNet(currentObjectNetPos + 1);
		}
	}

	/**
	 *
	 */
	void buttonCancelMouseUp() {
		canceled = true;
		RunAutoSimulationAction.setRunning(false);
		closeDialog();
	}

	/**
	 *
	 */
	void buttonApplyRuleMouseUp() {
		canceled = false;
		// store current match in a map
		closeDialog();
	}

	/**
	 *
	 */
	private void closeDialog() {
		// remove listener
		currentObjectNet.eAdapters().remove(this);

		// delete all mappings that have been created in this dialog so far
		deleteCurrentMatch();
		dialogShell.close();
	}

	void buttonCalculateAndResetUp() {
		// check if we just switched to completion mode
		if (matchCompletions == null) {
			// prepare completion mode
			matchCompletions = computeMatchCompletions();
			// is there any completion at all?
			if (matchCompletions.isEmpty()) {
				deleteCurrentMatch();
				textMessage.setText("No matches found!");
			} else {
				// apply first match completion
				setMatchCompletionAsCurrent(0);
			}
			buttonCalculateAndReset.setText("Reset matches");
			buttonCalculateAndReset
					.setToolTipText("Discard the current match and all completions");
		} else {
			resetMatchCompletions();
		}
	}

	/***************************************************************************
	 * Getters to retrieve information after dialog has been closed.
	 **************************************************************************/

	/**
	 * @return the match being cached 1. by pushing apply button 2. for current
	 *         match checking 3. for computing match completions
	 * 
	 * @see #cacheCurrentMatch()
	 */
	public NetMorphism getMatchToApply() {
		return currentMatch;
	}

	/**
	 * @return the current object net
	 */
	public ObjectNet getCurrentObjectNet() {
		return currentObjectNet;
	}

	/**
	 * @return the currentRule
	 * 
	 * @see #setCurrentRule(int)
	 */
	public Rule getCurrentRule() {
		return currentRule;
	}

	/**
	 * @return <code>true</code> if this dialog has been canceled.
	 */
	public boolean isCanceled() {
		return canceled;
	}
}
