package roneditor.gef.actions.ron;

import java.util.ArrayList;
import java.util.List;

import muvitorkit.animation.AnimatingCommand;
import muvitorkit.dialogs.AbstractMatchDialog;
import muvitorkit.ui.MuvitorTreeEditor;

import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.ui.IWorkbenchPart;

import roneditor.gef.commands.ron.CopyTokenToHLPlaceCommand;
import roneditor.gef.commands.ron.DeleteTokenCommand;
import roneditor.gef.editparts.ron.HLTransitionEditPart;
import roneditor.model.HLTransitionUtil;
import roneditor.model.ObjectNetUtil;
import roneditor.ui.dialogs.MatchDialog;
import ronmodel.HLPlace;
import ronmodel.HLTransition;
import ronmodel.HLTransitionKind;
import ronmodel.ObjectNet;
import ronmodel.RON;
import ronmodel.Rule;
import ronmodel.Token;
import aggSim.Grammar;
import aggSim.NetMorphism;

/**
 * This is a sample action to demonstrate AbstractMatchDialog. It is enabled if
 * the RON in the background of the tree viewer is selected. It just grabs all
 * rules and object nets in the RON and puts the result of the rule application
 * on the HLPlace that hosts the selected object net.
 * 
 * @author Tony Modica
 */
public class ApplyRuleDialogAction extends SelectionAction {

	public static final String ID = "roneditor.dialogs.match.ApplyRuleDialogAction";

	private static final String NAME = "Match Dialog";

	public ApplyRuleDialogAction(final IWorkbenchPart part) {
		super(part);
		setId(ID);
		setText(NAME);
	}

	@Override
	public void run() {
		final HLTransitionEditPart transitionEditpart = (HLTransitionEditPart) getSelectedObjects()
				.get(0);
		final HLTransition transition = transitionEditpart.getCastedModel();

		final List<ObjectNet> sourceTokens = HLTransitionUtil
				.collectSourceNetTokens(transition);
		final ObjectNet unionObjectNet = ObjectNetUtil
				.unionIntoObjectNet(sourceTokens);

		final List<HLPlace> incomingRulePlaces = HLTransitionUtil
				.getIncomingRulePlaces(transition);

		// get copies of all rules and object nets in the RON
		final ArrayList<Rule> rulesCopies = new ArrayList<Rule>();
		for (final Token token : incomingRulePlaces.get(0).getTokens()) {
			rulesCopies.add((Rule) token);
		}
		final ArrayList<ObjectNet> objectNets = new ArrayList<ObjectNet>();
		objectNets.add(unionObjectNet);

		// open dialog
		final AbstractMatchDialog dialog = new MatchDialog(rulesCopies,
				objectNets);
		dialog.open();
		if (dialog.isCanceled()) {
			return;
		}

		// fetch information from the dialog
		final Rule selectedRule = dialog.getCurrentRule();
		// final Rule selectedRule = (Rule)
		// incomingRulePlaces.get(0).getTokens()
		// .get(rulesCopies.indexOf(selectedRuleCopy));
		final ObjectNet selectedObjectNetCopy = dialog.getCurrentObjectNet();
		final NetMorphism matchOnCopy = dialog.getMatchToApply();

		final CompoundCommand compoundCommand = new CompoundCommand();

		// muvitorkit.animation: move objectnets and rule from pre-domain places
		// to
		// transition, done without overwriting initialize() because at this
		// time all elements to be animated exist but could have done so ntl.
		final RON ron = transition.getRon();
		AnimatingCommand aniComm = new AnimatingCommand();
		for (final ObjectNet token : sourceTokens) {
			aniComm.initializeAnimatedElement(token, ron);
			aniComm.specifyStep(token, token.getPlace());
		}
		aniComm.initializeAnimatedElement(selectedRule, ron);
		aniComm.specifyStep(selectedRule, selectedRule.getPlace());
		aniComm.nextStep();
		for (final ObjectNet token : sourceTokens) {
			aniComm.specifyStep(token, transition);
		}
		aniComm.specifyStep(selectedRule, transition);
		compoundCommand.add(aniComm);

		// remove tokens/ObjectNets from place
		for (final ObjectNet t : sourceTokens) {
			compoundCommand.add(new DeleteTokenCommand(t));
			// close views that show the removed token object net(s)
			MuvitorTreeEditor.closeViewShowing(t);
		}

		// remove rule from the HLPlace of Type RULE
		compoundCommand.add(new DeleteTokenCommand(selectedRule));
		// close views that show the removed token rule
		MuvitorTreeEditor.closeViewShowing(selectedRule);

		// apply the selected rule to the selected object net
		new Grammar().applyRule(matchOnCopy, selectedRule,
				selectedObjectNetCopy);

		// Copy rule to the new outgoing tokens on HLPlace.RULE
		final List<HLPlace> outgoingRulePlaces = HLTransitionUtil
				.getOutgoingRulePlaces(transition);
		for (final HLPlace place : outgoingRulePlaces) {
			compoundCommand.add(new CopyTokenToHLPlaceCommand(selectedRule,
					place, false));
		}

		// Copy unionObjectNet to the new outgoing tokens on HLPlace.NET
		final List<HLPlace> outgoingNetPlaces = HLTransitionUtil
				.getOutgoingNetPlaces(transition);
		for (final HLPlace place : outgoingNetPlaces) {
			compoundCommand.add(new CopyTokenToHLPlaceCommand(
					selectedObjectNetCopy, place, false));
		}

		// muvitorkit.animation: move rule and new unionObjectNets from
		// transition to
		// post-domain places, done by overwriting initialize() because elements
		// to be animated will not exist before compound command is being
		// executed!
		aniComm = new AnimatingCommand() {
			@Override
			public void initialize() {
				// the last tokens on outgoingNet/RulePlaces should be the
				// tokens to
				// be animated, since they have been put there by the previously
				// executed command
				// define transition as starting point for result net copies
				for (final HLPlace place : outgoingNetPlaces) {
					final EList<Token> tokens = place.getTokens();
					final Token token = tokens.get(tokens.size() - 1);
					initializeAnimatedElement(token, ron);
					specifyStep(token, transition);
				}
				// define transition as muvitorkit.animation starting point for
				// copies of
				// the used rule
				for (final HLPlace place : outgoingRulePlaces) {
					final EList<Token> tokens = place.getTokens();
					final Token token = tokens.get(tokens.size() - 1);
					initializeAnimatedElement(token, ron);
					specifyStep(token, transition);
				}
				nextStep();
				// define their actual place as muvitorkit.animation target
				// point for
				// copies of the used rule
				for (final HLPlace place : outgoingNetPlaces) {
					final EList<Token> tokens = place.getTokens();
					final Token token = tokens.get(tokens.size() - 1);
					specifyStep(token, place);
				}
				// define their actual place as animantion target point for
				// copies of the used rule
				for (final HLPlace place : outgoingRulePlaces) {
					final EList<Token> tokens = place.getTokens();
					final Token token = tokens.get(tokens.size() - 1);
					specifyStep(token, place);
				}
			}
		};
		compoundCommand.add(aniComm);

		getCommandStack().execute(compoundCommand);
	}

	/**
	 * The {@link SelectionAction} is enabled if the first selected object is a
	 * {@link HLTransition} of kind <i>APPLYRULE</i>.
	 */
	@Override
	protected boolean calculateEnabled() {
		if (getSelection() != null && getSelectedObjects().size() == 1) {
			if (getSelectedObjects().get(0) instanceof HLTransitionEditPart) {
				final HLTransitionEditPart transitionEditpart = (HLTransitionEditPart) getSelectedObjects()
						.get(0);
				final HLTransition t = transitionEditpart.getCastedModel();
				if (t.getKind().getValue() == HLTransitionKind.APPLYRULE
						&& t.isActivated()) {
					return true;
				}
			}
		}
		return false;
	}

}
