package roneditor.gef.commands.ron;

import java.util.ArrayList;
import java.util.List;

import muvitorkit.animation.AnimatingCommand;

import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.commands.CompoundCommand;

import roneditor.model.HLTransitionUtil;
import roneditor.model.ObjectNetUtil;
import ronmodel.HLPlace;
import ronmodel.HLTransition;
import ronmodel.ObjectNet;
import ronmodel.RON;
import ronmodel.Rule;
import ronmodel.Token;
import aggSim.Grammar;

/**
 * Perform <code>APPLYRULE</code> operation on the {@link HLTransition}.
 */
public class HLTransitionApplyRuleCommand extends CompoundCommand {
	private static final String LABEL = "Apply Rule '";
	private final HLTransition transition;
	private final boolean couldApplyRule;

	public HLTransitionApplyRuleCommand(final HLTransition transition) {
		this.transition = transition;
		// Collect all NET tokens to move/combine
		// also consider multiple tokens from the same place
		final List<ObjectNet> tokens = HLTransitionUtil
				.collectSourceNetTokens(transition);
		final ObjectNet unionObjectNet = ObjectNetUtil
				.unionIntoObjectNet(tokens);

		// Collect incoming NET/RULE HLPlaces
		final List<HLPlace> incomingRulePlaces = HLTransitionUtil
				.getIncomingRulePlaces(transition);
		final Rule rule = (Rule) incomingRulePlaces.get(0).getTokens().get(0);

		setLabel(LABEL + rule.getName() + "' on HLTrans '"
				+ transition.getName() + "'");

		/*
		 * animation: move objectnets and rule from pre-domain places to
		 * transition, done without overwriting initialize() because at this
		 * time all elements to be animated exist (but could have done so ntl.)
		 */
		final RON ron = transition.getRon();
		final AnimatingCommand aniComm = new AnimatingCommand();
		aniComm.setCompleteDuration(1000);
		for (final ObjectNet token : tokens) {
			aniComm.initializeAnimatedElement(token, ron);
			aniComm.specifyStep(token, token.getPlace());
		}
		aniComm.initializeAnimatedElement(rule, ron);
		aniComm.specifyStep(rule, rule.getPlace());
		aniComm.nextStep();
		for (final ObjectNet token : tokens) {
			aniComm.specifyStep(token, transition);
		}
		aniComm.specifyStep(rule, transition);
		add(aniComm);

		// apply rule to unionObjectNet
		couldApplyRule = new Grammar().applyRule(rule, unionObjectNet);

		// remove rule from the HLPlace of Type RULE
		add(new DeleteTokenCommand(rule));

		// remove tokens/ObjectNets from place
		for (final ObjectNet t : tokens) {
			add(new DeleteTokenCommand(t));
		}

		// Copy rule to the new outgoing tokens on HLPlace.RULE
		final List<HLPlace> outgoingRulePlaces = HLTransitionUtil
				.getOutgoingRulePlaces(transition);
		for (final HLPlace place : outgoingRulePlaces) {
			add(new CopyTokenToHLPlaceCommand(rule, place, false));
		}

		// Copy unionObjectNet to the new outgoing tokens on HLPlace.NET
		final List<HLPlace> outgoingNetPlaces = HLTransitionUtil
				.getOutgoingNetPlaces(transition);
		for (final HLPlace place : outgoingNetPlaces) {
			add(new CopyTokenToHLPlaceCommand(unionObjectNet, place, false));
		}

		/*
		 * muvitorkit.animation: move rule and new unionObjectNets from
		 * transition to post-domain places, done by overwriting initialize()
		 * because elements to be animated will not exist before compound
		 * command is being executed
		 */
		final AnimatingCommand aniComm2 = new AnimatingCommand() {
			@Override
			public void initialize() {
				setCompleteDuration(1000);
				// the last tokens on outgoingPlaces should be the tokens to
				// be animated
				final List<HLPlace> outgoingPlaces = new ArrayList<HLPlace>();
				outgoingPlaces.addAll(outgoingNetPlaces);
				outgoingPlaces.addAll(outgoingRulePlaces);

				for (final HLPlace place : outgoingPlaces) {
					final EList<Token> outTokens = place.getTokens();
					final Token token = outTokens.get(outTokens.size() - 1);
					initializeAnimatedElement(token, ron);
					specifyStep(token, transition);
				}
				nextStep();
				for (final HLPlace place : outgoingPlaces) {
					final EList<Token> outTokens = place.getTokens();
					final Token token = outTokens.get(outTokens.size() - 1);
					specifyStep(token, place);
				}
			}
		};
		add(aniComm2);
	}

	@Override
	public void execute() {
		// before animating close views showing object nets that have been
		// removed
		HLTransitionUtil.closeViewsForConsumedObjectNets(transition);
		super.execute();
	}

	@Override
	public boolean canExecute() {
		return couldApplyRule;
	}

}
