package roneditor.gef.editparts.objectnet;

import java.util.ArrayList;
import java.util.List;

import muvitorkit.gef.editparts.AdapterGraphicalEditPart;
import muvitorkit.ui.utils.SWTResourceManager;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.ConnectionLayer;
import org.eclipse.draw2d.FanRouter;
import org.eclipse.draw2d.FreeformLayer;
import org.eclipse.draw2d.FreeformLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.ShortestPathConnectionRouter;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartListener;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.editpolicies.RootComponentEditPolicy;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.gef.editpolicies.RONXYLayoutEditPolicy;
import roneditor.model.RuleUtil;
import roneditor.model.properties.ObjectNetPropertySource;
import ronmodel.Mapping;
import ronmodel.ObjectNet;
import ronmodel.RonmodelPackage;

/**
 * EditPart for the main component {@link ObjectNet}
 */
public class ObjectNetEditPart extends AdapterGraphicalEditPart<ObjectNet> {

	private Label nameLabel;

	private final EditPartListener editPartListener = new EditPartListener.Stub() {
		/**
		 * Called when the state of the selection is changed. Simply refreshes
		 * the visuals to show if the EditPart is selected.
		 */
		@Override
		public void selectedStateChanged(final EditPart part) {
			refreshVisuals();
		}
	};

	/**
	 * Constructor
	 * 
	 * @param net
	 *            the {@link ObjectNet}
	 */
	public ObjectNetEditPart(final ObjectNet net) {
		super(net);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	@Override
	protected IFigure createFigure() {
		final FreeformLayer layer = new FreeformLayer();
		layer.setLayoutManager(new FreeformLayout());

		// use a fancy ShortestPathConnectionRouter with FanRouter
		final ConnectionLayer cLayer = (ConnectionLayer) getLayer(LayerConstants.CONNECTION_LAYER);
		cLayer.setAntialias(SWT.ON);
		final FanRouter frouter = new FanRouter();
		final ShortestPathConnectionRouter router = new ShortestPathConnectionRouter(
				layer);
		frouter.setSeparation(20);
		router.setSpacing(20);
		frouter.setNextRouter(router);
		cLayer.setConnectionRouter(frouter);

		nameLabel = new Label();
		nameLabel.setFont(SWTResourceManager.getFont("Sans", 14, SWT.BOLD));
		nameLabel.setForegroundColor(Display.getCurrent().getSystemColor(
				SWT.COLOR_GRAY));
		nameLabel.setText(getCastedModel().getName());
		layer.add(nameLabel, new Rectangle(10, 10, -1, -1));

		return layer;
	}

	/**
	 * Install the EditPolicies for the EditPart. In particular we want to have
	 * the {@link ObjectNetEditPartGraphLayoutEditPolicy} assigned.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	@Override
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new RONXYLayoutEditPolicy());
		installEditPolicy(EditPolicy.NODE_ROLE, null);
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE, null);
		installEditPolicy(EditPolicy.SELECTION_FEEDBACK_ROLE, null);
		installEditPolicy(EditPolicy.COMPONENT_ROLE,
				new RootComponentEditPolicy());
	}

	/**
	 * Return all children of the Model
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	@Override
	protected List<EObject> getModelChildren() {
		final List<EObject> netChildren = new ArrayList<EObject>();
		netChildren.addAll(getCastedModel().getNodes());
		return netChildren;
	}

	/**
	 * When activated the listener for changes on the model is registered and a
	 * {@link Label} showing the name of the {@link ObjectNet} is shown in the
	 * upper left corner of the {@link EditPart}
	 * 
	 * @see muvitorkit.gef.editparts.AdapterGraphicalEditPart#activate()
	 */
	@Override
	public void activate() {
		// Listen for changes of the selection state
		addEditPartListener(editPartListener);

		refreshVisuals();
		super.activate();

		RuleUtil.updateColors(getCastedModel());
	}

	/**
	 * Update the figure. If the EditPart is selected, the border will be
	 * thickened.
	 */
	@Override
	protected void refreshVisuals() {
		final FreeformLayer layer = (FreeformLayer) getFigure();
		getFigure().setBackgroundColor(ColorConstants.white);
		if (getSelected() == EditPart.SELECTED_NONE) {
			layer.setBorder(new LineBorder(1));
			layer.setForegroundColor(SWTResourceManager.getColor(0, 0, 0));
		} else { // selected
			layer.setBorder(new LineBorder(2));
			layer.setForegroundColor(SWTResourceManager.getColor(150, 0, 0));
		}
		getFigure().repaint();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see muvitorkit.gef.editparts.AdapterGraphicalEditPart#deactivate()
	 */
	@Override
	public void deactivate() {
		removeEditPartListener(editPartListener);
		super.deactivate();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * muvitorkit.gef.editparts.AdapterGraphicalEditPart#createPropertySource()
	 */
	@Override
	protected IPropertySource createPropertySource() {
		return new ObjectNetPropertySource(getCastedModel());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * muvitorkit.gef.editparts.AdapterGraphicalEditPart#notifyChanged(org.eclipse
	 * .emf.common.notify.Notification)
	 */
	@Override
	public void notifyChanged(final Notification notification) {
		final int featureId = notification.getFeatureID(RonmodelPackage.class);
		// handle problem that notifications from ONPlaces are forwarded to
		// object nets and featureIDs may overlap (e.g. CODOMAIN__MAPPINGS ==
		// ON_NODE__INCOMING)
		if (notification.getNotifier() instanceof ObjectNet) {
			switch (featureId) {
			case RonmodelPackage.OBJECT_NET__NAME:
				nameLabel.setText(getCastedModel().getName());
				refreshVisuals();
				break;
			case RonmodelPackage.OBJECT_NET__NODES:
			case RonmodelPackage.OBJECT_NET__ARCS:
				refreshChildren();
				break;
			case RonmodelPackage.OBJECT_NET__MAPPINGS:
				// if a mapping is added to or removed from the object net
				// update the colors of the nodes for this mapping
				Mapping mapping = (Mapping) notification.getNewValue();
				if (mapping == null) {
					// handle the case that a mapping has been removed
					mapping = (Mapping) notification.getOldValue();
				}
				if (mapping != null) {
					RuleUtil.updateColor(mapping.getSource());
					RuleUtil.updateColor(mapping.getTarget());
				}
				break;
			default:
				break;
			}
		}
	}
}
