package roneditor.gef.editparts.objectnet.policies;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;

import roneditor.gef.commands.rule.UpdateMappingCommand;
import roneditor.gef.editparts.objectnet.ONNodeEditPart;
import roneditor.gef.figures.objectnet.NodeFigure;
import ronmodel.LHS;
import ronmodel.Mapping;
import ronmodel.NAC;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ObjectNet;
import ronmodel.RHS;

/**
 * {@link GraphicalNodeEditPolicy} for the {@link ONNodeEditPart}. Provides some
 * convenience functions and initiates the feedback and connection handling of
 * {@link Mapping} commands.
 */
public class ONNodeEditPolicy extends GraphicalNodeEditPolicy {

	/**
	 * Return the casted model of the host {@link EditPart}.
	 * 
	 * @return
	 */
	protected ONNode getCastedModel() {
		return (ONNode) getHost().getModel();
	}

	/**
	 * If we create a {@link Mapping} we want to have another feedback. This
	 * method is called while the connection is not finished.
	 */
	@Override
	public void showSourceFeedback(final Request request) {
		if (request instanceof CreateConnectionRequest) {
			final CreateConnectionRequest createConnectionRequest = (CreateConnectionRequest) request;
			final Object requestingObject = createConnectionRequest
					.getNewObject();
			if (requestingObject instanceof Mapping) {
				if (!(getCastedModel().getObjectNet() instanceof LHS)) {
					return;
				}
				((NodeFigure) getHostFigure()).setGlow(true);

				return;
			}
		}
		super.showSourceFeedback(request);
	}

	/**
	 * If we have create a {@link Mapping} we want to have another feedback.
	 * This method is called when the connection is finished.
	 */
	@Override
	public void eraseSourceFeedback(final Request request) {
		if (request instanceof CreateConnectionRequest) {
			final CreateConnectionRequest createConnectionRequest = (CreateConnectionRequest) request;
			final Object requestingObject = createConnectionRequest
					.getNewObject();
			if (requestingObject instanceof Mapping) {
				if (!(getCastedModel().getObjectNet() instanceof LHS)) {
					return;
				}
				((NodeFigure) getHostFigure()).setGlow(false);
				return;
			}
		}
		super.eraseSourceFeedback(request);
	}

	/**
	 * Called whenever the {@link CreateConnectionRequest} requests the
	 * completion {@link Command}. When our start {@link Command} is an
	 * {@link UpdateMappingCommand} we update the {@link Codomain} and target
	 * model appropriately.
	 */
	@Override
	protected Command getConnectionCompleteCommand(
			final CreateConnectionRequest request) {
		final Command command = request.getStartCommand();
		if (command instanceof UpdateMappingCommand) {
			final ObjectNet objectNet = getCastedModel().getObjectNet();
			if (request.getExtendedData().get("MatchDialogMode") == null
					&& !(objectNet instanceof RHS || objectNet instanceof NAC)) {
				return null;
			}
			final UpdateMappingCommand createMappingCommand = (UpdateMappingCommand) command;
			createMappingCommand.setCodomain(objectNet);
			createMappingCommand.setTarget(getCastedModel());
			createMappingCommand.update();
			return command;
		}
		return null;
	}

	/**
	 * Called whenever the {@link CreateConnectionRequest} requests the creation
	 * {@link Command}. Handles the creation of the {@link UpdateMappingCommand}
	 * when we want to create a {@link Mapping}.
	 */
	@Override
	protected Command getConnectionCreateCommand(
			final CreateConnectionRequest request) {
		final Object requestingObject = request.getNewObject();
		if (requestingObject instanceof Mapping) {
			if (!(getCastedModel().getObjectNet() instanceof LHS)) {
				return null;
			}
			UpdateMappingCommand command = (UpdateMappingCommand) request
					.getStartCommand();
			if (command == null) {
				command = new UpdateMappingCommand();
				command.setMapping((Mapping) requestingObject);
			}
			command.setSource(getCastedModel());
			command.update();
			request.setStartCommand(command);
			return command;
		}

		return null;
	}

	@Override
	protected Command getReconnectSourceCommand(final ReconnectRequest request) {
		return null;
	}

	@Override
	protected Command getReconnectTargetCommand(final ReconnectRequest request) {
		return null;
	}

	/**
	 * Helper method to avoid doubly connected nodes.
	 * 
	 * @param source
	 *            A node to test if it is already connected to this EditParts
	 *            model
	 * @return <code>true</code> if source is already connected to this
	 *         EditParts model
	 */
	protected boolean isConnectedByArc(final ONNode source) {
		final Object hostModel = getHost().getModel();
		for (final ONArc arc : source.getOutgoing()) {
			if (arc.getTarget() == hostModel) {
				return true;
			}
		}
		return false;
	}

}
