package roneditor.gef.editparts.ron;

import java.util.List;

import muvitorkit.gef.directedit.IDirectEditPart.IGraphicalDirectEditPart;
import muvitorkit.gef.editparts.AdapterGraphicalEditPart;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.eclipse.jface.viewers.ICellEditorValidator;

import roneditor.gef.editparts.ron.policies.HLNodeComponentEditPolicy;
import roneditor.gef.figures.objectnet.NodeFigure;
import ronmodel.HLArc;
import ronmodel.HLNode;
import ronmodel.RonmodelPackage;

/**
 * EditPart for a generic HLNode. The derived {@link HLTransitionEditPart} and
 * {@linkHL PlaceEditPolicy} need to implement the deviant functionality.
 */
abstract public class HLNodeEditPart<T extends HLNode> extends
		AdapterGraphicalEditPart<T> implements NodeEditPart,
		IGraphicalDirectEditPart {

	/**
	 * Creates a new {@link HLNodeEditPart} instance.
	 * 
	 * @param node
	 *            The Node
	 */
	public HLNodeEditPart(final T node) {
		super(node);
	}

	/**
	 * Get the incoming connections of the node
	 * 
	 * @return List of HLNodes
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelSourceConnections()
	 */
	@Override
	protected List<HLArc> getModelSourceConnections() {
		return getCastedModel().getOutgoing();
	}

	/**
	 * Get the incoming connections of the node
	 * 
	 * @return List of HLNodes
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelTargetConnections()
	 */
	@Override
	protected List<HLArc> getModelTargetConnections() {
		return getCastedModel().getIncoming();
	}

	/**
	 * Returns the Figure of this, as a transition type figure.
	 * 
	 * @return Figure as a TransitionFigure.
	 */
	protected NodeFigure getNodeFigure() {
		return (NodeFigure) getFigure();
	}

	/**
	 * Create all basic policies for a HLNode.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	@Override
	protected void createEditPolicies() {
		// install the edit policy to handle deletion
		installEditPolicy(EditPolicy.COMPONENT_ROLE,
				new HLNodeComponentEditPolicy());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef
	 * .ConnectionEditPart)
	 */
	@Override
	public ConnectionAnchor getSourceConnectionAnchor(
			final ConnectionEditPart connection) {
		return getNodeFigure().getSourceConnectionAnchor();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef
	 * .Request)
	 */
	@Override
	public ConnectionAnchor getSourceConnectionAnchor(final Request request) {
		return getNodeFigure().getSourceConnectionAnchor();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef
	 * .ConnectionEditPart)
	 */
	@Override
	public ConnectionAnchor getTargetConnectionAnchor(
			final ConnectionEditPart connection) {
		return getNodeFigure().getTargetConnectionAnchor();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef
	 * .Request)
	 */
	@Override
	public ConnectionAnchor getTargetConnectionAnchor(final Request request) {
		return getNodeFigure().getTargetConnectionAnchor();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	@Override
	protected void refreshVisuals() {
		getNodeFigure().updateValueDisplay(getCastedModel().getName());
		final Point loc = new Point(getCastedModel().getX(), getCastedModel()
				.getY());
		final Dimension size = new Dimension(-1, -1);
		final Rectangle r = new Rectangle(loc, size);
		((GraphicalEditPart) getParent()).setLayoutConstraint(this,
				getFigure(), r);
		getNodeFigure().validate();
		super.refreshVisuals();
	}

	@Override
	protected void notifyChanged(final Notification msg) {
		final int featureId = msg.getFeatureID(RonmodelPackage.class);
		switch (featureId) {
		case RonmodelPackage.HL_NODE__NAME:
			refreshVisuals();
			break;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see muvitorkit.gef.directedit.IDirectEditPart#getDirectEditFeatureID()
	 */
	@Override
	public int getDirectEditFeatureID() {
		return RonmodelPackage.HL_NODE__NAME;
	}

	@Override
	public ICellEditorValidator getDirectEditValidator() {
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seemuvitorkit.gef.directedit.IDirectEditPart.IGraphicalDirectEditPart#
	 * getValueLabelTextBounds()
	 */
	@Override
	public Rectangle getValueLabelTextBounds() {
		return getNodeFigure().getValueLabelTextBounds();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seemuvitorkit.gef.directedit.IDirectEditPart.IGraphicalDirectEditPart#
	 * updateValueDisplay(java.lang.String)
	 */
	@Override
	public void updateValueDisplay(final String value) {
		getNodeFigure().updateValueDisplay(value);
	}
}
