package roneditor.gef.editparts.ron;

import muvitorkit.gef.directedit.IDirectEditPart.IGraphicalDirectEditPart;
import muvitorkit.gef.directedit.MuvitorDirectEditPolicy;
import muvitorkit.gef.editparts.AdapterGraphicalEditPart;
import muvitorkit.gef.editparts.policies.IGhostFigureProvider;
import muvitorkit.ui.MuvitorActivator;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.FlowLayout;
import org.eclipse.draw2d.FreeformLayeredPane;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LabeledContainer;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.draw2d.parts.Thumbnail;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.ui.parts.GraphicalViewerImpl;
import org.eclipse.jface.viewers.ICellEditorValidator;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.RONIconUtil;
import roneditor.gef.editparts.objectnet.ObjectNetEditPartFactory;
import roneditor.gef.editparts.ron.policies.TokenComponentEditPolicy;
import roneditor.model.properties.ObjectNetPropertySource;
import roneditor.model.properties.RulePropertySource;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.NAC;
import ronmodel.ObjectNet;
import ronmodel.RonmodelPackage;
import ronmodel.Rule;
import ronmodel.Token;

/**
 */
public class SimpleTokenEditPart extends AdapterGraphicalEditPart<Token>
		implements IGraphicalDirectEditPart {

	/**
	 * @param token
	 */
	public SimpleTokenEditPart(final Token token) {
		super(token);
	}

	/**
	 * Create a {@link Thumbnail} of the given {@link ObjectNet} of the given
	 * preferred {@link Dimension}.
	 * 
	 * @param net
	 * @param dimension
	 * @return The {@link Figure} of the {@link ObjectNet}
	 */
	private Figure createThumbnail(final ObjectNet net, final int height) {
		final GraphicalViewer viewer = new GraphicalViewerImpl();
		viewer.createControl(PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getShell());
		final ScalableFreeformRootEditPart rootEditPart = new ScalableFreeformRootEditPart();
		viewer.setRootEditPart(rootEditPart);
		viewer.setEditPartFactory(new ObjectNetEditPartFactory());
		viewer.setContents(net);
		viewer.flush();
		final Thumbnail thumbnail = new Thumbnail();
		final FreeformLayeredPane source = (FreeformLayeredPane) rootEditPart
				.getLayer(LayerConstants.PRINTABLE_LAYERS);
		thumbnail.setSource(source);
		final Dimension sourceSize = source.getFreeformExtent().getSize();
		sourceSize.scale(height / sourceSize.preciseHeight());
		thumbnail.setSize(sourceSize);
		thumbnail.setVisible(true);
		return thumbnail;
	}

	/**
	 * Create a {@link Thumbnail} of the {@link Token} that can be used as a
	 * tool tip.
	 * 
	 * @return the thumbnail figure displaying the token net or rule
	 */
	Figure getThumbnail() {
		final LabeledContainer thumbnailContainer = new LabeledContainer();
		// Use horizontal FlowLayout
		final FlowLayout layout = new FlowLayout(true);
		thumbnailContainer.setLayoutManager(layout);
		// The default height for each thumbnail
		final int height = 300;
		if (getCastedModel() instanceof Rule) {
			thumbnailContainer.setLabel(getCastedModel().getName());
			final Rule rule = (Rule) getCastedModel();

			// Add the NACs if there are some
			for (final NAC nac : rule.getNacs()) {
				final Figure nacFigure = createThumbnail(nac, height);
				thumbnailContainer.add(nacFigure);
			}

			// Add the LHS if there are any nodes on it
			if (!rule.getLhs().getNodes().isEmpty()) {
				final Figure lhs = createThumbnail(rule.getLhs(), height);
				thumbnailContainer.add(lhs);
			}
			// Add the RHS if there are any nodes on it
			if (!rule.getRhs().getNodes().isEmpty()) {
				final Figure rhs = createThumbnail(rule.getRhs(), height);
				thumbnailContainer.add(rhs);
			}
		} else {
			// The dimension for the object net
			thumbnailContainer.setLabel("");
			final Figure objectNet = createThumbnail((ObjectNet) getModel(),
					height);
			thumbnailContainer.add(objectNet);
		}
		return thumbnailContainer;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * muvitorkit.gef.editparts.AdapterGraphicalEditPart#createPropertySource()
	 */
	@Override
	protected IPropertySource createPropertySource() {
		if (getCastedModel() instanceof Rule) {
			return new RulePropertySource((Rule) getCastedModel());
		}
		return new ObjectNetPropertySource((ObjectNet) getCastedModel());
	}

	/**
	 * Create an Label as a Figure with it's type specific icon depending on the
	 * {@link HLPlaceType}.
	 */
	@Override
	protected IFigure createFigure() {
		final SimpleTokenLabel label = new SimpleTokenLabel();
		label.setText(getCastedModel().getName());
		label.setIcon(RONIconUtil.getIcon(RonmodelPackage.TOKEN,
				getCastedModel().getPlace().getType().getValue(), 16));
		return label;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	@Override
	protected void refreshVisuals() {
		// high-light the token if it is first in the parent place
		final HLPlace place = getCastedModel().getPlace();

		if (place == null || getParent() == null) {
			// let's see if this happens again
			MuvitorActivator.logError(
					"place or parent is null for SimpleTokenEditPart", null);
			return;
		}

		// highlight if is first token on place
		if (place.getTokens().get(0) == getCastedModel()) {
			getFigure().setForegroundColor(ColorConstants.black);
		} else {
			getFigure().setForegroundColor(ColorConstants.gray);
		}

		((Label) getFigure()).setText(getCastedModel().getName());
		super.refreshVisuals();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	@Override
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.COMPONENT_ROLE,
				new TokenComponentEditPolicy());
		installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE,
				new MuvitorDirectEditPolicy());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * muvitorkit.gef.editparts.AdapterGraphicalEditPart#notifyChanged(org.eclipse
	 * .emf.common.notify.Notification)
	 */
	@Override
	public void notifyChanged(final Notification notification) {
		final int featureId = notification.getFeatureID(RonmodelPackage.class);
		switch (featureId) {
		case RonmodelPackage.TOKEN__NAME:
		case RonmodelPackage.TOKEN__PLACE:
			refreshVisuals();
			break;
		}
	}

	/**
	 *
	 */
	class SimpleTokenLabel extends Label implements IGhostFigureProvider {

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.draw2d.Figure#getToolTip()
		 */
		@Override
		public IFigure getToolTip() {
			if (super.getToolTip() == null) {
				setToolTip(getThumbnail());
			}
			return super.getToolTip();
		}

		@Override
		public IFigure getGhostFigure() {
			final Label ghostLabel = new Label(getText(), getIcon());
			ghostLabel.setForegroundColor(ColorConstants.gray);
			// ghostLabel.setBackgroundColor(ghostFillColor);
			return ghostLabel;
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see muvitorkit.gef.directedit.IDirectEditPart#getDirectEditFeatureID()
	 */
	@Override
	public int getDirectEditFeatureID() {
		return RonmodelPackage.TOKEN__NAME;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see muvitorkit.gef.directedit.IDirectEditPart#getDirectEditValidator()
	 */
	@Override
	public ICellEditorValidator getDirectEditValidator() {
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seemuvitorkit.gef.directedit.IDirectEditPart.IGraphicalDirectEditPart#
	 * getValueLabelTextBounds()
	 */
	@Override
	public Rectangle getValueLabelTextBounds() {
		return ((SimpleTokenLabel) getFigure()).getTextBounds();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seemuvitorkit.gef.directedit.IDirectEditPart.IGraphicalDirectEditPart#
	 * updateValueDisplay(java.lang.String)
	 */
	@Override
	public void updateValueDisplay(final String value) {
		((SimpleTokenLabel) getFigure()).setText(value);
	}
}
