package roneditor.gef.editparts.ron.policies;

import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;

import roneditor.gef.commands.ron.ConnectHLNodesCommand;
import roneditor.gef.commands.ron.ReconnectHLNodeCommand;
import roneditor.gef.editparts.objectnet.ONTransitionEditPart;
import roneditor.gef.editparts.ron.HLTransitionEditPart;
import roneditor.model.HLTransitionUtil;
import ronmodel.HLArc;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.HLTransition;
import ronmodel.ONTransition;

/**
 * {@link EditPolicy} for an {@link HLTransitionEditPart}.
 */
public class HLTransitionEditPolicy extends HLNodeEditPolicy {

	@Override
	protected HLTransition getCastedModel() {
		return (HLTransition) super.getCastedModel();
	}

	/**
	 * When ever a connection is completed, we update the provided
	 * {@link CreateConnectionRequest} with the target, hence the
	 * {@link ONTransition} of the {@link ONTransitionEditPart}.
	 * 
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCompleteCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	@Override
	protected Command getConnectionCompleteCommand(
			final CreateConnectionRequest request) {
		// HLTransition can have at most one RULE-typed place in its predomain
		final Object sourceModel = request.getSourceEditPart().getModel();
		if (sourceModel instanceof HLPlace) {
			final HLPlace sourcePlace = (HLPlace) sourceModel;
			if (sourcePlace.getType() == HLPlaceType.RULE_LITERAL
					&& hasIncomingRulePlace()) {
				return null;
			}
		}

		final Command command = request.getStartCommand();
		if (command instanceof ConnectHLNodesCommand) {
			final ConnectHLNodesCommand connectHLNodesCommand = (ConnectHLNodesCommand) command;
			if (connectHLNodesCommand.isSourcePlace()
					&& !isConnectedByArc(connectHLNodesCommand.getSource())) {
				connectHLNodesCommand.setTarget(getCastedModel());
				return connectHLNodesCommand;
			}
		}

		return super.getConnectionCompleteCommand(request);
	}

	/**
	 * When ever a connection is created, we update the provided
	 * {@link CreateConnectionRequest} with the source, hence the
	 * {@link ONTransition} of the {@link ONTransitionEditPart}.
	 * 
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */

	@Override
	protected Command getConnectionCreateCommand(
			final CreateConnectionRequest request) {
		final Object requestingObject = request.getNewObject();
		if (requestingObject instanceof HLArc) {
			final ConnectHLNodesCommand command = new ConnectHLNodesCommand(
					getCastedModel(), (HLArc) requestingObject);
			request.setStartCommand(command);
			return command;
		}
		return super.getConnectionCreateCommand(request);
	}

	/**
	 * When ever a connection is changed, we update the provided
	 * {@link ReconnectRequest} with the target, hence the {@link ONTransition}
	 * of the {@link ONTransitionEditPart}.
	 * 
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	@Override
	protected Command getReconnectTargetCommand(final ReconnectRequest request) {
		final HLArc link = (HLArc) request.getConnectionEditPart().getModel();
		if (link.getSource() instanceof HLPlace
				&& link.getTarget() != getCastedModel()) {
			// HLTransition can have at most one RULE-typed place in its
			// predomain
			final HLPlace sourcePlace = (HLPlace) link.getSource();
			if (sourcePlace.getType() == HLPlaceType.RULE_LITERAL
					&& hasIncomingRulePlace()) {
				return null;
			}
			final ReconnectHLNodeCommand command = new ReconnectHLNodeCommand(
					link, sourcePlace, getCastedModel());
			return command;
		}
		return null;
	}

	/**
	 * When ever a connection is changed, we update the provided
	 * {@link ReconnectRequest} with the source, hence the {@link ONTransition}
	 * of the {@link ONTransitionEditPart}.
	 * 
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	@Override
	protected Command getReconnectSourceCommand(final ReconnectRequest request) {
		final HLArc link = (HLArc) request.getConnectionEditPart().getModel();
		if (link.getTarget() instanceof HLPlace
				&& link.getSource() != getCastedModel()) {
			final ReconnectHLNodeCommand command = new ReconnectHLNodeCommand(
					link, getCastedModel(), link.getTarget());
			return command;
		}
		return null;
	}

	private boolean hasIncomingRulePlace() {
		return !HLTransitionUtil.getIncomingRulePlaces(getCastedModel())
				.isEmpty();
	}

}
