package roneditor.model;

import java.util.ArrayList;
import java.util.List;

import muvitorkit.ui.utils.GenericUtils;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ONPlace;
import ronmodel.ONTransition;

/**
 * This Class provides some basic (static) operations for {@link ONTransition}
 * not to be found in the generated model.
 */
public class ONTransitionUtil {

	private ONTransitionUtil() {
	}

	/**
	 * Check if the given {@link ONTransition} can fire. Hence if all source
	 * places have enough tokens and if all target places have sufficient
	 * capacity.
	 * 
	 * @param transition
	 *            the transition to check.
	 * @return <code>true</code> if the ONTransition can fire.
	 */
	public static boolean canFire(final ONTransition transition) {
		// Check if all source places have enough tokens
		for (final ONArc incomingArc : transition.getIncoming()) {
			if (incomingArc == null) {
				return false;
			}
			final ONPlace sourcePlace = (ONPlace) incomingArc.getSource();
			if (sourcePlace == null) {
				return false;
			}
			if (sourcePlace.getTokens() < incomingArc.getWeight()) {
				return false;
			}
		}

		// Check if the capacity of all target places is sufficient
		for (final ONArc outgoingArc : transition.getOutgoing()) {
			if (outgoingArc == null) {
				return false;
			}
			final ONPlace targetPlace = (ONPlace) outgoingArc.getTarget();
			if (targetPlace == null) {
				return false;
			}
			final int newCapacity = outgoingArc.getWeight()
					+ targetPlace.getTokens();
			if (targetPlace.getCapacity() != 0
					&& targetPlace.getCapacity() < newCapacity) {
				return false;
			}
		}
		return true;
	}

	public static ONTransition getRandomActivated(final List<ONNode> nodes) {
		return GenericUtils.getRandom(getActivated(nodes));
	}

	public static List<ONTransition> getActivated(final List<ONNode> nodes) {
		final List<ONTransition> activated = new ArrayList<ONTransition>();
		for (final ONNode node : nodes) {
			if (node instanceof ONTransition && canFire((ONTransition) node)) {
				activated.add((ONTransition) node);
			}
		}
		return activated;
	}

}
