package roneditor.model;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import muvitorkit.ui.MuvitorActivator;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;

import ronmodel.LHS;
import ronmodel.Mapping;
import ronmodel.NAC;
import ronmodel.NodeColor;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ObjectNet;
import ronmodel.RHS;
import ronmodel.RonmodelFactory;
import ronmodel.Rule;
import ronmodel.impl.NodeColorImpl;

/**
 * Helper Class for {@link Rule} specific operations. Partly provides
 * functionality for the view in form of a color association of mappings.
 */

public final class RuleUtil {

	private RuleUtil() {
	}

	/**
	 * Return all mappings for a rule
	 * 
	 * @param rule
	 * @return List of all mappings that are in the rule
	 */
	public static final List<Mapping> getAllCompleteMappings(final Rule rule) {
		final List<Mapping> mappings = new ArrayList<Mapping>();
		for (final Mapping mapping : rule.getRhs().getMappings()) {
			/*
			 * OPT muss das hier sein? Kann der Check nicht nur bei
			 * vollstndigen Mappings ausgefhrt werden? Ist in RulePage
			 * angepasst, testen!
			 */
			if (mapping.getSource() != null && mapping.getTarget() != null) {
				mappings.add(mapping);
			} else {
				MuvitorActivator.logError("Incomplete Mapping in RHS found!",
						null);
			}
		}
		for (final NAC nac : rule.getNacs()) {
			for (final Mapping mapping : nac.getMappings()) {
				// filter incomplete mappings
				if (mapping.getSource() != null && mapping.getTarget() != null) {
					mappings.add(mapping);
				} else {
					MuvitorActivator.logError(
							"Incomplete Mapping in NAC found!", null);
				}
			}
		}
		return mappings;
	}

	public static final boolean isPartOfRule(final ObjectNet net,
			final Rule rule) {
		return getContainingRule(net) == rule;
	}

	public static final Rule getContainingRule(final EObject model) {
		ObjectNet net;
		if (model instanceof ObjectNet) {
			net = (ObjectNet) model;
		} else if (model instanceof ONNode || model instanceof ONArc) {
			net = (ObjectNet) model.eContainer();
		} else {
			return null;
		}

		if (net instanceof LHS) {
			return ((LHS) net).getRule();
		}
		if (net instanceof RHS) {
			return ((RHS) net).getRule();
		}
		if (net instanceof NAC) {
			return ((NAC) net).getRule();
		}
		return null;
	}

	/***************************************************************************
	 * Neuer Versuch fr Coloring von Mappings
	 **************************************************************************/

	private static final NodeColor defaultColor = RonmodelFactory.eINSTANCE
			.createNodeColor();

	private static final NodeColor createDefaultColor() {
		return RonmodelFactory.eINSTANCE.createNodeColor();
	}

	public static final NodeColor determineColorFor(final ONNode node) {
		if (getIncidentMappings(node).isEmpty()) {
			return createDefaultColor();
		}
		final NodeColor transitiveColor = findTransitiveColor(node,
				new HashSet<ONNode>());
		if (transitiveColor != null) {
			// System.out.println("Transitive '" + transitiveColor + "' to '" +
			// node.getObjectNet().getName() + "' node '" + node.getName());
			return transitiveColor;
		}
		return createNewColor(node);
	}

	private static final NodeColor findTransitiveColor(final ONNode node,
			final Set<ONNode> checkedNodes) {
		// return the first Color being found in the "mapping graph"!
		if (checkedNodes.add(node)) {
			for (final Mapping mapping : getIncidentMappings(node)) {
				final ONNode opposite = getOppositeNode(mapping, node);
				final NodeColor opColor = opposite.getColor();
				if (opColor != null && !opColor.equals(defaultColor)) {
					// System.out.print("Found transitive color on node " +
					// opposite.getName() + "@" +
					// opposite.getObjectNet().getName() + " -> ");
					// FIXED needed to copy the color. Otherwise the old owner's
					// color would be set to "null"
					return EcoreUtil.copy(opColor);
				}
				final NodeColor transitiveOpColor = findTransitiveColor(
						opposite, checkedNodes);
				if (transitiveOpColor != null) {
					return transitiveOpColor;
				}
			}
		}
		return null;
	}

	private static final NodeColor createNewColor(final ONNode node) {
		final List<NodeColor> forbiddenColors = new ArrayList<NodeColor>();
		collectForbiddenColors(node.getObjectNet(), new HashSet<ObjectNet>(),
				forbiddenColors);
		NodeColor color;
		int c = 1;
		do {
			final float f = (float) Math.ceil(c / 360.0f);
			// step in value direction
			color = new NodeColorImpl(c % 360, 0.7f / f, 1.0f);
			/*
			 * we may try one of these nice prime-numbers too 2, 3, 5, 7, 11,
			 * 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61, 67, 71, 73,
			 * 79, 83, 89, 97, 101, 103, 107, 109, 113
			 */
			c += 47; // step in hue direction
		} while (forbiddenColors.contains(color));
		// System.out.println("Assigned '" + color + "' to '" +
		// node.getObjectNet().getName() + "' node '" + getName(node) + "',
		// forbidden: " + forbiddenColors);
		return color;
	}

	private static final void collectForbiddenColors(final ObjectNet objectNet,
			final Set<ObjectNet> foundNets, final List<NodeColor> forbidden) {
		for (final ONNode node : objectNet.getNodes()) {
			// remember all colors used by nodes in the current object net
			final NodeColor color = node.getColor();
			if (color != null && !color.equals(defaultColor)
					&& !forbidden.contains(color)) {
				forbidden.add(color);
			}
			// proceed with all object nets reachable from a mapping on a node
			// in this net
			for (final Mapping mapping : getIncidentMappings(node)) {
				final ONNode opposite = getOppositeNode(mapping, node);
				final ObjectNet oppositeNet = opposite.getObjectNet();
				// search net that have not been searched only
				if (foundNets.add(oppositeNet)) {
					collectForbiddenColors(oppositeNet, foundNets, forbidden);
				}
			}
		}
	}

	/**
	 * Method to update the color of a node
	 * 
	 * @param node
	 */
	public static final void updateColor(final ONNode node) {
		final NodeColor c = determineColorFor(node);
		node.setColor(c);
	}

	/**
	 * Convenience method for updating colors in whole nets
	 * 
	 * @param objectNet
	 *            the net to be updated
	 */
	public static final void updateColors(final ObjectNet objectNet) {
		for (final ONNode node : objectNet.getNodes()) {
			updateColor(node);
		}
	}

	/**
	 * Convenience method for updating colors in whole rules
	 * 
	 * @param rule
	 *            the rule to be updated
	 */
	public static final void updateColors(final Rule rule) {
		updateColors(rule.getLhs());
		updateColors(rule.getRhs());
		for (final NAC nac : rule.getNacs()) {
			updateColors(nac);
		}
	}

	/**
	 * Unregisters the colors for the passe node if it has no incident mappings
	 * other that the passed one.
	 * 
	 * @param node
	 * @param exclude
	 */
	public static final void resetColor(final ONNode node, final Mapping exclude) {
		final List<Mapping> mappings = getIncidentMappings(node);
		mappings.remove(exclude);
		if (mappings.isEmpty()) {
			node.setColor(createDefaultColor());
		}
	}

	/**
	 * Unregisters the colors for each node in the passed objectNet
	 * 
	 * @param objectNet
	 */
	public static final void resetColors(final ObjectNet objectNet) {
		for (final ONNode node : objectNet.getNodes()) {
			node.setColor(createDefaultColor());
		}
	}

	/**
	 * Unregisters the colors for each node in the passed rule
	 * 
	 * @param rule
	 */
	public static final void resetColors(final Rule rule) {
		resetColors(rule.getLhs());
		resetColors(rule.getRhs());
		for (final NAC nac : rule.getNacs()) {
			resetColors(nac);
		}
	}

	public static final List<Mapping> getIncidentMappings(final ONNode node) {
		final List<Mapping> result = new ArrayList<Mapping>();
		// filter incomplete mappings and temporary matches
		for (final Mapping mapping : node.getIsSourceInMappings()) {
			if (mapping.getTarget() != null && mapping.getCodomain() != null) {
				result.add(mapping);
			}
		}
		for (final Mapping mapping : node.getIsTargetInMappings()) {
			if (mapping.getSource() != null && mapping.getCodomain() != null) {
				result.add(mapping);
			}
		}
		return result;
	}

	public static final ONNode getOppositeNode(final Mapping mapping,
			final ONNode node) {
		if (mapping.getSource() == node) {
			return mapping.getTarget();
		}
		if (mapping.getTarget() == node) {
			return mapping.getSource();
		}
		return null;
	}

}