package roneditor.ui.rule;

import java.util.List;

import muvitorkit.gef.palette.MuvitorPaletteRoot;
import muvitorkit.ui.ContextMenuProviderWithActionRegistry;
import muvitorkit.ui.MuvitorPage;
import muvitorkit.ui.MuvitorPageBookView;
import muvitorkit.ui.utils.MuvitorNotifierService;
import muvitorkit.ui.utils.MuvitorNotifierService.IChangeListener;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.KeyHandler;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.part.IPageSite;

import roneditor.gef.actions.objectnet.FireONTransitionAction;
import roneditor.gef.actions.rule.CopyLHSToNACAction;
import roneditor.gef.actions.rule.CopyLHSToRHSAction;
import roneditor.gef.actions.rule.CopyRHSToNACAction;
import roneditor.gef.actions.rule.CreateNACAction;
import roneditor.gef.actions.rule.DeleteNACAction;
import roneditor.gef.actions.rule.SwitchNACAction;
import roneditor.gef.editparts.objectnet.ObjectNetEditPartFactory;
import roneditor.model.RuleChecker;
import roneditor.model.RuleUtil;
import ronmodel.LHS;
import ronmodel.NAC;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ONPlace;
import ronmodel.ObjectNet;
import ronmodel.RHS;
import ronmodel.RonmodelPackage;
import ronmodel.Rule;

/**
 * <p>
 * This is an implementation of {@link MuvitorPageBookView} to display
 * {@link Rule}s.
 * </p>
 * 
 * <p>
 * It uses {@link ObjectNetEditPartFactory} and
 * {@link RulePageContextMenuProvider}, configures the key-handler and creates
 * actions to move {@link ONNode}s by pressing keys.
 * </p>
 * 
 * <p>
 * On model change nothing is done.
 * </p>
 */
public class RulePage extends MuvitorPage implements IChangeListener {

	public RulePage(final MuvitorPageBookView view) {
		super(view);
	}

	private NAC activeNAC;

	@Override
	public void init(final IPageSite pageSite) {
		super.init(pageSite);
		MuvitorNotifierService.addListener(this, getEditor());
		RuleUtil.resetColors(getRule());
	}

	@Override
	public void dispose() {
		MuvitorNotifierService.removeListener(this);
		super.dispose();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * roneditor.objectnetview.AbstractPageWithFlyoutPalette#createActions()
	 */
	@Override
	protected void createCustomActions() {
		registerSharedActionAsHandler(ActionFactory.COPY.getId());
		registerSharedActionAsHandler(ActionFactory.CUT.getId());
		registerSharedActionAsHandler(ActionFactory.PASTE.getId());
		// A sample action to switch through the NACs
		registerAction(new SwitchNACAction(this));
		registerAction(new CreateNACAction(this, getEditor()));
		registerAction(new DeleteNACAction(this, getEditor()));
		registerAction(new CopyLHSToRHSAction(this, getEditor()));
		registerAction(new CopyLHSToNACAction(this, getEditor()));
		registerAction(new CopyRHSToNACAction(this, getEditor()));
		registerActionAsHandler(new FireONTransitionAction(getEditor()));
		// registerSharedAction(ExportPNMLAction.ID);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * roneditor.objectnetview.AbstractPageWithFlyoutPalette#setupKeyHandler
	 * (org.eclipse.gef.KeyHandler)
	 */
	@Override
	protected void setupKeyHandler(final KeyHandler kh) {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * roneditor.objectnetview.AbstractPageWithFlyoutPalette#createEditPartFactory
	 * ()
	 */
	@Override
	protected EditPartFactory createEditPartFactory() {
		// Just show the nets like in ObjectNetView, nothing special here
		return new ObjectNetEditPartFactory();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * roneditor.objectnetview.AbstractPageWithFlyoutPalette#getContextMenuProvider
	 * ()
	 */
	@Override
	protected ContextMenuProviderWithActionRegistry createContextMenuProvider(
			final EditPartViewer viewer) {
		return new RulePageContextMenuProvider(viewer);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * roneditor.objectnetview.AbstractPageWithFlyoutPalette#createPaletteRoot()
	 */
	@Override
	protected MuvitorPaletteRoot createPaletteRoot() {
		return new RulePaletteRoot();
	}

	/**
	 * The Adapter is registered to all critical areas for a rule, hence we
	 * react on model changes that need the morphisms be rechecked for validity
	 * or mark our color table for the mappings dirty.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * muvitorkit.ui.MuvitorPage#notifyChanged(org.eclipse.emf.common.notify
	 * .Notification)
	 */
	@Override
	public void notifyChanged(final Notification notification) {
		final int featureId = notification.getFeatureID(RonmodelPackage.class);
		final int eventId = notification.getEventType();
		// ensure we have a visible NAC
		if (featureId == RonmodelPackage.RULE__NACS
				&& (eventId == Notification.ADD || eventId == Notification.REMOVE)) {
			setViewerVisibility(0, getActiveNAC() != null);
			setViewersContents(0, getActiveNAC());
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * muvitorkit.ui.utils.GlobalNotifierService.IChangeListener#notifyChange
	 * (org.eclipse.emf.ecore.EObject,
	 * org.eclipse.emf.common.notify.Notification)
	 */
	@Override
	public void notifyChange(final EObject model, final Notification msg) {
		final int featureId = msg.getFeatureID(RonmodelPackage.class);
		// On critical model changes we need to re-validate the morphisms
		if (model instanceof ONNode
				&& RuleUtil.isPartOfRule(((ONNode) model).getObjectNet(),
						getRule())) {
			switch (featureId) {
			case RonmodelPackage.ON_NODE__NAME:
				RuleChecker.isRuleOk(getRule());
				break;
			}
			if (model instanceof ONPlace) {
				switch (featureId) {
				case RonmodelPackage.ON_PLACE__TOKENS:
					RuleChecker.isRuleOk(getRule());
					break;
				}
			}
		}
		if (model instanceof ONArc
				&& RuleUtil.isPartOfRule(((ONArc) model).getObjectNet(),
						getRule())) {
			switch (featureId) {
			case RonmodelPackage.ON_ARC__SOURCE:
			case RonmodelPackage.ON_ARC__TARGET:
			case RonmodelPackage.ON_ARC__WEIGHT:
				// only check complete mappings (when reconnecting etc.)
				final ONArc arc = (ONArc) model;
				if (arc.getSource() != null && arc.getTarget() != null) {
					RuleChecker.isRuleOk(getRule());
				}
				break;
			}
		}
		if (model instanceof ObjectNet
				&& RuleUtil.isPartOfRule((ObjectNet) model, getRule())) {
			switch (featureId) {
			case RonmodelPackage.OBJECT_NET__PLACE:
			case RonmodelPackage.OBJECT_NET__NODES:
			case RonmodelPackage.OBJECT_NET__ARCS:
			case RonmodelPackage.OBJECT_NET__MAPPINGS:
				RuleChecker.isRuleOk(getRule());
				break;
			}
		}

	}

	/**
	 * A {@link RulePage} shows the first {@link NAC} of the {@link Rule} and
	 * the {@link LHS} and {@link RHS}.
	 * 
	 * @see muvitorkit.ui.MuvitorPage#getViewerContents()
	 */
	@Override
	public EObject[] getViewerContents() {
		final Rule rule = (Rule) getModel();
		final LHS lhs = rule.getLhs();
		final RHS rhs = rule.getRhs();
		return new EObject[] { getActiveNAC(), lhs, rhs };
	}

	/**
	 * @return The model of the page casted to a {@link Rule}.
	 */
	public Rule getRule() {
		return (Rule) getModel();
	}

	/**
	 * Change the contents of the viewer for the NACs (the left-most) to the
	 * next element in the list of the rule's NACs. If the last NAC is being
	 * shown switch to the first in the list.
	 */
	public void setNextNAC() {
		final List<NAC> nacs = getRule().getNacs();
		final NAC newNAC;
		if (getActiveNAC() == null) {
			newNAC = null;
		} else {
			final int actualShowing = nacs.lastIndexOf(getActiveNAC());
			newNAC = nacs.get((actualShowing + 1) % nacs.size());
		}
		setActiveNAC(newNAC);
	}

	/**
	 * Return the active {@link NAC} with some initial checking. If we have no
	 * {@link NAC}s associated return <code>null</code>, if we have no active
	 * {@link NAC} yet, set the first one from the list of {@link NAC}s as
	 * active {@link NAC}. Return then active {@link NAC}.
	 * 
	 * @return <code>null</code> if there is no {@link NAC}, the active
	 *         {@link NAC} otherwise (the first one as default)
	 */
	public NAC getActiveNAC() {
		if (getRule().getNacs().size() == 0) {
			return null;
		} else if (activeNAC == null) {
			activeNAC = getRule().getNacs().get(0);
		}
		return activeNAC;
	}

	public void setActiveNAC(final NAC activeNAC) {
		this.activeNAC = activeNAC;
		setViewersContents(0, activeNAC);
		setViewerVisibility(0, activeNAC != null);
	}
}
